unit DateEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MyProc, Buttons, ExtCtrls, commctrl, Menus, Spin, DBCtrls, DB, DBTables, DsgnIntf;

const
  InitRepeatPause = 400;  { pause before repeat timer (ms) }
  RepeatPause     = 100;  { pause before hint window displays (ms)}
  AccelerateArray: array[0..10] of Byte = (1,2,5,10,15,20,25,30,40,50,100);

type
  TDateType = (dtDateSingle, dtDateRange);
  TGridOption = (goViewToDay, goViewWeekNumber, goSingleDate);
  TGridOptions = set of TGridOption;
  TDateSpeedButton = class;

  TDateGrid = class(TWinControl)
  private
    FCanvas: TCanvas;
    FCtl3D: Boolean;
    FCurDay: TPoint;
    FCurrentDate: TDate;
    FDecMonth: TDateSpeedButton;
    FDecYear: TDateSpeedButton;
    FEllipse: TBitmap;
    FFirstDate: TDate;
    FGridOptions: TGridOptions;
    FHSpacing: Integer;
    FIncMonth: TDateSpeedButton;
    FIncYear: TDateSpeedButton;
    FLabel: TLabel;
    FLangType: TLangType;
    FLastDate: TDate;
    FLeftClick: Boolean;
    FOldDay: TPoint;
    FOnMouseLeave: TNotifyEvent;
    FMenuItems: array[1..12] of TMenuItem;
    FMonthPopup: TPopupMenu;
    FToDay: TBitmap;
    FVSpacing: Integer;
    FWorkDate: TDate;
    FYearEdit: TSpinEdit;
    FYearLabel: TLabel;
    FOldWD, FOldFD, FOldLD: TDate;
    FUpSpace, FLeftSpace: Integer;
    procedure ButtonsClick(Sender: TObject);
    procedure ChangeMonth(Sender: TObject);
    procedure CheckDateRange(Sender: TObject);
    procedure CheckMenuPopup(const Pos: TSmallPoint);
    function  CreateButton(X, Y: Integer; BitmapName: string; Increment: Integer): TDateSpeedButton;
    procedure DoMouseDown(var Message: TWMMouse; Button: TMouseButton; Shift: TShiftState);
    procedure DoMouseUp(var Message: TWMMouse; Button: TMouseButton);
    procedure Draw;
    procedure DrawCurrentDay;
    procedure DrawEllipse(Date: TDate);
    procedure DrawFrame;
    procedure DrawWorkDate(Color: TColor);
    function  IsDateFirst(Date, FD: TDate): Boolean;
    function  IsDateInRange(Date, FD, LD: TDate): Boolean;
    function  IsDateLast(Date, LD: TDate): Boolean;
    procedure LabelClick(Sender: TObject);
    procedure YearLabelClick(Sender: TObject);
    procedure YearEditExit(Sender: TObject);
    function  RepaintThisDate(CD, OD: TDate; i,j: Integer): Boolean;
    procedure SetCtl3D(Value: Boolean);
    procedure SetFirstDate(Value: TDate);
    procedure SetGridOptions(Value: TGridOptions);
    procedure SetHSpacing(Value: Integer);
    procedure SetLangType(Value: TLangType);
    procedure SetLastDate(Value: TDate);
    procedure SetVSpacing(Value: Integer);
    procedure SetWorkDate(Value: TDate);
    procedure CMMouseLeave(var Message: TNotifyEvent); message CM_MOUSELEAVE;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMNCLButtonDown(var Message: TWMNCLButtonDown); message WM_NCLBUTTONDOWN;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
    procedure WMMButtonDown(var Message: TWMMButtonDown); message WM_MBUTTONDOWN;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure WMRButtonDblClk(var Message: TWMRButtonDblClk); message WM_RBUTTONDBLCLK;
    procedure WMMButtonDblClk(var Message: TWMMButtonDblClk); message WM_MBUTTONDBLCLK;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure WMRButtonUp(var Message: TWMRButtonUp); message WM_RBUTTONUP;
    procedure WMMButtonUp(var Message: TWMMButtonUp); message WM_MBUTTONUP;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
  protected
    procedure Click; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure MouseControl(Shift: TShiftState; X, Y: Integer; Click: Boolean);
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; virtual;
    procedure PaintWindow(DC: HDC); override;
    property  Canvas: TCanvas read FCanvas;
    property WorkDate: TDate read FWorkDate write SetWorkDate;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D default True;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FirstDate: TDate read FFirstDate write SetFirstDate;
    property Options: TGridOptions read FGridOptions write SetGridOptions default [goSingleDate, goViewToDay];
    property HSpacing: Integer read FHSpacing write SetHSpacing default 25;
    property LangType: TLangType read FLangType write SetLangType default ltEnglish;
    property LastDate: TDate read FLastDate write SetLastDate;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property VSpacing: Integer read FVSpacing write SetVSpacing default 17;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnMouseDown;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property OnStartDrag;
  end;

  TNumGlyphs = Buttons.TNumGlyphs;

  TDateButton = class (TWinControl)
  private
    FUpButton: TDateSpeedButton;
    FDownButton: TDateSpeedButton;
    FFocusedButton: TDateSpeedButton;
    FFocusControl: TWinControl;
    FOnUpClick: TNotifyEvent;
    FOnDownClick: TNotifyEvent;
    function  CreateButton: TDateSpeedButton;
    function  GetUpGlyph: TBitmap;
    function  GetDownGlyph: TBitmap;
    procedure SetUpGlyph(Value: TBitmap);
    procedure SetDownGlyph(Value: TBitmap);
    function  GetUpNumGlyphs: TNumGlyphs;
    function  GetDownNumGlyphs: TNumGlyphs;
    procedure SetUpNumGlyphs(Value: TNumGlyphs);
    procedure SetDownNumGlyphs(Value: TNumGlyphs);
    procedure BtnClick(Sender: TObject);
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SetFocusBtn (Btn: TDateSpeedButton);
    procedure AdjustSize (var W: Integer; var H: Integer);
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  protected
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Align;
    property Ctl3D;
    property DownGlyph: TBitmap read GetDownGlyph write SetDownGlyph;
    property DownNumGlyphs: TNumGlyphs read GetDownNumGlyphs write SetDownNumGlyphs default 1;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl: TWinControl read FFocusControl write FFocusControl;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property UpGlyph: TBitmap read GetUpGlyph write SetUpGlyph;
    property UpNumGlyphs: TNumGlyphs read GetUpNumGlyphs write SetUpNumGlyphs default 1;
    property Visible;
    property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDrag;
    property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
  end;

  TDateIncrement = (diDay, diMonth, diYear);

  TCustomDateEdit = class(TCustomEdit)
  private
    FBtnDownX: Integer;
    FButton: TDateButton;
    FCurrentDate: TDate;
    FDateButton: TDateSpeedButton;
    FDateForm: TForm;
    FDateGrid: TDateGrid;
    FDateIncrement: TDateIncrement;
    FDateType: TDateType;
    FEditorEnabled: Boolean;
    FFirstDate: TDate;
    FFormVisible: Boolean;
    FLangType: TLangType;
    FLastDate: TDate;
    FMask: string;
    FMouseLeave: TNotifyEvent;
    FMouseEnter: TNotifyEvent;
    FPosition: Integer;
    FWorkDate: TDate;
    procedure DeleteKeys(CharCode: Word);
    procedure FormCreate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    function  GetMinHeight: Integer;
    procedure GridClick(Sender: TObject);
    procedure HomeEndKeys(CharCode: Word; Shift: TShiftState);
    procedure ChangeDate(Sender: TObject);
    function  CharKeys(var CharCode: Char): Boolean;
    procedure CheckCursor;
    procedure CheckDate(Sender: TObject);
    procedure CheckValidDate;
    procedure LeftRightKeys(CharCode: Word; Shift: TShiftState);
    procedure SetDateIncrement(Value: TDateIncrement);
    procedure SetDateType(Value: TDateType);
    procedure SetEditRect;
    procedure SetFirstDate(Value: TDate);
    procedure SetLangType(Value: TLangType);
    procedure SetLastDate(Value: TDate);
    procedure SetWorkDate(Value: TDate);
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CMMouseEnter (var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Msg: TMessage); message CM_MOUSELEAVE;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DateButtonClick(Senter: TObject);
    procedure DoChange; virtual;
    procedure DownClick(Senter: TObject);
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    function  IsValidChar(Key: Char): Boolean; virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure UpClick(Senter: TObject);
    property WorkDate: TDate read FWorkDate write SetWorkDate;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetSel(var SelStart: Integer; var SelStop: Integer);
    procedure SetSel(SelStart: Integer; SelStop: Integer);
    property CurrentDate: TDate read FCurrentDate;
    property DateIncrement: TDateIncrement read FDateIncrement write SetDateIncrement;
    property DateType: TDateType read FDateType write SetDateType;
    property EditorEnabled: Boolean read FEditorEnabled write FEditorEnabled default True;
    property FirstDate: TDate read FFirstDate write SetFirstDate;
    property LangType: TLangType read FLangType write SetLangType;
    property LastDate: TDate read FLastDate write SetLastDate;
    property OnMouseEnter: TNotifyEvent read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TNotifyEvent read FMouseLeave write FMouseLeave;
  end;

  TDateEdit = class(TCustomDateEdit)
  published
    property AutoSelect;
    property AutoSize;
    property Color;
//    property Ctl3D;
    property DateIncrement;
    property DateType;
    property DragCursor;
    property DragMode;
    property EditorEnabled;
    property Enabled;
    property FirstDate;
    property Font;
    property LangType;
    property LastDate;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TDBDateEdit = class(TCustomDateEdit)
  private
    FDataLink: TFieldDataLink;
    FCanvas: TControlCanvas;
    FAlignment: TAlignment;
    FFocused: Boolean;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function  GetDataField: string;
    function  GetDataSource: TDataSource;
    function  GetField: TField;
    function  GetReadOnly: Boolean;
//    function  GetTextMargins: TPoint;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetFocused(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
//    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    procedure Change; override;
    procedure DoChange; override;
    function  EditCanModify: Boolean; virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Reset; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property AutoSelect;
    property AutoSize;
    property Color;
//    property Ctl3D;
    property DateIncrement;
    property DragCursor;
    property DragMode;
    property EditorEnabled;
    property Enabled;
    property FirstDate;
    property Font;
    property LangType;
    property LastDate;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TDateBtnState = set of (tbFocusRect, tbAllowTimer);

  TDateSpeedButton = class(TSpeedButton)
  private
    FRepeatTimer: TTimer;
    FDateBtnState: TDateBtnState;
    FAccelerate: Boolean;
    FPosInArray: Byte;
    FIncrement: Byte;
    FCount: Byte;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DateBtnState: TDateBtnState read FDateBtnState write FDateBtnState;
    property Accelerate: Boolean read FAccelerate write FAccelerate default False;
    property Increment: Byte read FIncrement;
  end;

  TFieldProperty = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

procedure Register;

implementation

{$R DateEdit}

uses Clipbrd;

procedure Register;
begin
  RegisterComponents('A-Team', [TDateEdit]);
  RegisterComponents('A-Team', [TDBDateEdit]);
//  RegisterComponents('A-Team', [TDateButton]);
  RegisterComponents('A-Team', [TDateGrid]);
  RegisterPropertyEditor(TypeInfo(string),TDBDateEdit,'DataField',TFieldProperty);
end;


{DateGrid}

constructor TDateGrid.Create(AOwner: TComponent);
var i: Integer;
begin
  inherited Create(AOwner);
  Color:=clWhite;
  FUpSpace:=50; FLeftSpace:=7;
  HSpacing:=25;
  VSpacing:=17;
  FGridOptions:=[goViewToDay, goSingleDate];
  FCanvas := TControlCanvas.Create;
  TControlCanvas(FCanvas).Control := Self;
  FCurrentDate:=TDate.Create;
  FFirstDate:=TDate.Create;
  FLastDate:=TDate.Create;
  FWorkDate:=TDate.Create;
  FOldWD:=TDate.Create;
  FOldFD:=TDate.Create;
  FOldLD:=TDate.Create;
  FFirstDate.OnChange:=CheckDateRange;
  FLastDate.OnChange:=CheckDateRange;
  FEllipse:=TBitmap.Create;
  FEllipse.Transparent:=True;
  FEllipse.Handle:=LoadBitmap(HInstance, 'ELLIPSE');
  FToDay:=TBitmap.Create;
  FToDay.Transparent:=True;
  FToDay.Handle:=LoadBitmap(HInstance, 'TODAY');
  FLabel:=TLabel.Create(self);
  FLabel.Parent:=Self;
  FLabel.Transparent:=True;
  FLabel.Caption:=FWorkDate.StrMonth;
  FLabel.Top:=7;
  FLabel.Font.Style:=[fsBold];
  FLabel.Font.Color:=ClWhite;
  FLabel.OnClick:=LabelClick;
  FYearLabel:=TLabel.Create(self);
  FYearLabel.Parent:=Self;
  FYearLabel.Transparent:=True;
  FYearLabel.Caption:=IntToStr(FWorkDate.Year);
  FYearLabel.Top:=7;
  FYearLabel.Font.Style:=[fsBold];
  FYearLabel.Font.Color:=ClWhite;
  FYearLabel.OnClick:=YearLabelClick;
  FYearEdit:=TSpinEdit.Create(self);
  FYearEdit.Parent:=Self;
  FYearEdit.MaxLength:=4;
  FYearEdit.Visible:=False;
  FYearEdit.OnExit:=YearEditExit;
  FYearEdit.Top:=3;
  FYearEdit.Left:=-50;
  FYearEdit.Width:=50;
  FIncMonth:=CreateButton(116,8,'INCMONTH',1);
  FDecMonth:=CreateButton(17,8,'DECMONTH',-1);
  FIncYear:=CreateButton(128,8,'INCYEAR',12);
  FDecYear:=CreateButton(5,8,'DECYEAR',-12);
  FLangType:=ltEnglish;
  ControlStyle := ControlStyle - [csSetCaption] + [csOpaque, csClickEvents, csFixedWidth, csFixedHeight];
  TabStop:=True;
  FOldDay:=Point(9,9);
  FCurDay:=Point(9,9);
  FLeftClick:=False;
  FMonthPopup:=TPopupMenu.Create(self);
  for i:=1 to 12 do
    begin
    FMenuItems[i]:=TMenuItem.Create(self);
    case FLangType of
      ltCzech: FMenuItems[i].Caption:=MonthCZ[i];
      else FMenuItems[i].Caption:=MonthEN[i];
      end;
    FMenuItems[i].Tag:=i;
    FMenuItems[i].OnClick:=ChangeMonth;
    FMonthPopup.Items.Add(FMenuItems[i]);
    end;
end;

destructor TDateGrid.Destroy;
var i: Integer;
begin
  for i:=1 to 12 do FMenuItems[i].Free;
  FMonthPopup.Free;
  FLabel.Free;
  FCurrentDate.Free;
  FFirstDate.Free;
  FLastDate.Free;
  FWorkDate.Free;
  FCanvas.Free;
  FIncMonth.Free;
  FDecMonth.Free;
  FIncYear.Free;
  FDecYear.Free;
  FYearLabel.Free;
  FYearEdit.Free;
  inherited Destroy;
end;

procedure TDateGrid.Loaded;
var i: Integer;
begin
  inherited;
  for i:=1 to 12 do case FLangType of
    ltCzech: FMenuItems[i].Caption:=MonthCZ[i];
    else FMenuItems[i].Caption:=MonthEN[i];
    end;
  FYearEdit.Ctl3D:=False;
  FYearEdit.Visible:=False;
  FWorkDate.Assign(FFirstDate);
  FOldWD.Assign(FWorkDate);
  FOldFD.Assign(FFirstDate);
  FOldLD.Assign(FLastDate);
end;

procedure TDateGrid.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
//  Params.Style := 0 or ES_MULTILINE or WS_CLIPCHILDREN;
  CreateSubClass(Params, WC_LISTVIEW);
end;

procedure TDateGrid.ChangeMonth(Sender: TObject);
begin
  FFirstDate.OnChange:=nil;
  FLastDate.OnChange:=nil;
  if FWorkDate.Month=TMenuItem(Sender).Tag then Exit;
  FWorkDate.Month:=TMenuItem(Sender).Tag;
  FFirstDate.Assign(FWorkDate);
  FLastDate.Assign(FWorkDate);
  FLabel.Caption:=FWorkDate.StrMonth;
  FFirstDate.OnChange:=CheckDateRange;
  FLastDate.OnChange:=CheckDateRange;
  Draw;
end;

procedure TDateGrid.LabelClick(Sender: TObject);
var tmpPoint: TPoint;
begin
  GetCursorPos(tmpPoint);
  if not Focused then SetFocus;
  FMonthPopup.Popup(tmpPoint.x,tmpPoint.y);
end;

procedure TDateGrid.YearLabelClick(Sender: TObject);
begin
  FYearLabel.Visible:=False;
  FYearEdit.Left:=FYearLabel.Left;
  FYearEdit.Value:=FWorkDate.Year;
  FYearEdit.Visible:=True;
  FYearEdit.SetFocus;
end;

procedure TDateGrid.YearEditExit(Sender: TObject);
begin
  FFirstDate.OnChange:=nil;
  FLastDate.OnChange:=nil;
  FWorkDate.Year:=FYearEdit.Value;
  FYearLabel.Visible:=True;
  FYearEdit.Visible:=False;
  FFirstDate.Assign(FWorkDate);
  FLastDate.Assign(FWorkDate);
  FFirstDate.OnChange:=CheckDateRange;
  FLastDate.OnChange:=CheckDateRange;
  Draw;
  if not Focused then SetFocus;
end;

procedure TDateGrid.ButtonsClick(Sender: TObject);
begin
  FFirstDate.OnChange:=nil;
  FLastDate.OnChange:=nil;
  if not Focused then SetFocus;
  if TSpeedButton(Sender).Tag<0 then FWorkDate.DecrementMonth(AbsInt(TDateSpeedButton(Sender).Tag)*TDateSpeedButton(Sender).Increment) else FWorkDate.IncrementMonth(TDateSpeedButton(Sender).Tag*TDateSpeedButton(Sender).Increment);
  FFirstDate.Assign(FWorkDate);
  FLastDate.Assign(FWorkDate);
  FFirstDate.OnChange:=CheckDateRange;
  FLastDate.OnChange:=CheckDateRange;
  Draw;
end;

function  TDateGrid.CreateButton(X, Y: Integer; BitmapName: string; Increment: Integer): TDateSpeedButton;
begin
  Result:=TDateSpeedButton.Create (Self);
  Result.Left:=X;
  Result.Top:=Y;
  Result.Width:=12;
  Result.Height:=12;
  Result.OnClick := ButtonsClick;
  Result.Visible := True;
  Result.Enabled := True;
  Result.DateBtnState := [tbAllowTimer];
  Result.Glyph.Handle := LoadBitmap(HInstance, PChar(BitmapName));
  Result.Tag:=Increment;
  Result.Accelerate:=True;
//  Result.ShowHint:=True;
  Result.Parent := Self;
end;

procedure TDateGrid.SetCtl3D(Value: Boolean);
begin
  if Value=FCtl3D then Exit;
  FCtl3D:=Value;
  DrawFrame;
end;

procedure TDateGrid.SetFirstDate(Value: TDate);
begin
  if Value=FFirstDate then Exit;
  FFirstDate.Assign(Value);
end;

procedure TDateGrid.SetLangType(Value: TLangType);
var tmpRect: TRect;
    i: Byte;
begin
  if Value=FLangType then Exit;
  FLangType:=Value;
  FWorkDate.LangType:=Value;
  FFirstDate.LangType:=Value;
  FLastDate.LangType:=Value;
  FCurrentDate.LangType:=Value;
  tmpRect:=Rect(2,FUpSpace-20,Width-2,Height-2);
  InvalidateRect(Handle,@tmpRect,True);
  for i:=1 to 12 do case FLangType of
    ltCzech: FMenuItems[i].Caption:=MonthCZ[i];
    else FMenuItems[i].Caption:=MonthEN[i];
    end;
end;

procedure TDateGrid.SetLastDate(Value: TDate);
begin
  if Value=FLastDate then Exit;
  FLastDate.Assign(Value);
end;

procedure TDateGrid.SetGridOptions(Value: TGridOptions);
begin
  if FGridOptions=Value then Exit;
  FGridOptions:=Value;
  if goViewWeekNumber in FGridOptions then FLeftSpace:=30 else FLeftSpace:=7;
  if Width<>FHSpacing*6+FLeftSpace then Width:=FHSpacing*7+FLeftSpace;
  Invalidate;
end;

procedure TDateGrid.SetHSpacing(Value: Integer);
begin
  if FHSpacing=Value then Exit;
  FHSpacing:=Value;
  if Width<>FHSpacing*6+FLeftSpace then Width:=FHSpacing*7+FLeftSpace;
  Invalidate;
end;

procedure TDateGrid.SetVSpacing(Value: Integer);
begin
  if FVSpacing=Value then Exit;
  FVSpacing:=Value;
  if Height<>FVSpacing*6+FUpSpace then Height:=FVSpacing*6+FUpSpace;
  Invalidate;
end;

procedure TDateGrid.SetWorkDate(Value: TDate);
begin
  if Value=FWorkDate then Exit;
  FWorkDate.Assign(Value);
end;

procedure TDateGrid.CheckDateRange(Sender: TObject);
var tmpDate: TDate;
begin
  if not (goSingleDate in FGridOptions){FDateType=dtDateRange} then
    begin
    if FFirstDate.Julian>FLastDate.Julian then
      begin
      tmpDate:=TDate.Create;
      tmpDate.Assign(FFirstDate);
      FFirstDate.Assign(FLastDate);
      FLastDate.Assign(tmpDate);
      tmpDate.Free;
      end;
    end
  else if FFirstDate.Julian<>FLastDate.Julian then FLastDate.Assign(FFirstDate);
  if (FFirstDate.Month<>FWorkDate.Month) or (FFirstDate.Year<>FWorkDate.Year) then FWorkDate.Assign(FFirstDate);
  Draw;
end;

procedure TDateGrid.WMSize(var Message: TWMSize);
begin
  inherited;
  FIncYear.Left:=Width-16;
  FIncMonth.Left:=FIncYear.Left-12;
  FLabel.Left:=(Width div 2)-((FLabel.Width+FYearLabel.Width) div 2);
  FYearLabel.Left:=(Width div 2)-((FLabel.Width+FYearLabel.Width) div 2)+FLabel.Width;
  HSpacing:=(Width-FLeftSpace) div 7;
  VSpacing:=(Height-FUpSpace) div 6;
end;

procedure TDateGrid.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  FWorkDate.Assign(FFirstDate);
  FOldWD.Assign(FWorkDate);
  DrawWorkDate(clGray);
end;

procedure TDateGrid.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Invalidate;
  FWorkDate.Assign(FFirstDate);
end;

procedure TDateGrid.WMPaint(var Message: TWMPaint);
begin
  PaintHandler(Message);
end;

procedure TDateGrid.DoMouseDown(var Message: TWMMouse; Button: TMouseButton; Shift: TShiftState);
begin
  if not (csNoStdEvents in ControlStyle) then
    begin
    with Message do MouseDown(Button, KeysToShiftState(Keys) + Shift, XPos, YPos);
    if Button=mbLeft then
      begin
      FLeftClick:=True;
      SetFocus;
      MouseControl(KeysToShiftState(message.Keys)+Shift,message.XPos,message.YPos,True);
      end;
    end;
end;

procedure TDateGrid.WMLButtonDown(var Message: TWMLButtonDown);
begin
  SendCancelMode(Self);
  if csCaptureMouse in ControlStyle then MouseCapture := True;
  if csClickEvents in ControlStyle then ControlState:=ControlState + [csClicked];
  DoMouseDown(Message, mbLeft, []);
end;

procedure TDateGrid.WMNCLButtonDown(var Message: TWMNCLButtonDown);
begin
  SendCancelMode(Self);
end;

procedure TDateGrid.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  SendCancelMode(Self);
  if csCaptureMouse in ControlStyle then MouseCapture := True;
  if csClickEvents in ControlStyle then DblClick;
  DoMouseDown(Message, mbLeft, [ssDouble]);
end;

procedure TDateGrid.WMRButtonDown(var Message: TWMRButtonDown);
begin
  DoMouseDown(Message, mbRight, []);
end;

procedure TDateGrid.WMRButtonDblClk(var Message: TWMRButtonDblClk);
begin
  DoMouseDown(Message, mbRight, [ssDouble]);
end;

procedure TDateGrid.WMMButtonDown(var Message: TWMMButtonDown);
begin
  DoMouseDown(Message, mbMiddle, []);
end;

procedure TDateGrid.WMMButtonDblClk(var Message: TWMMButtonDblClk);
begin
  DoMouseDown(Message, mbMiddle, [ssDouble]);
end;

procedure TDateGrid.DoMouseUp(var Message: TWMMouse; Button: TMouseButton);
begin
  if not (csNoStdEvents in ControlStyle) then
    begin
    with Message do MouseUp(Button, KeysToShiftState(Keys), XPos, YPos);
    end;
end;

procedure TDateGrid.WMLButtonUp(var Message: TWMLButtonUp);
begin
  if csCaptureMouse in ControlStyle then MouseCapture := False;
  if csClicked in ControlState then
    begin
    ControlState:=ControlState - [csClicked];
    if PtInRect(ClientRect, SmallPointToPoint(Message.Pos)) then Click;
    end;
  DoMouseUp(Message, mbLeft);
  FLeftClick:=False;
end;

procedure TDateGrid.WMRButtonUp(var Message: TWMRButtonUp);
begin
  DoMouseUp(Message, mbRight);
  if Message.Result = 0 then CheckMenuPopup(Message.Pos);
end;

procedure TDateGrid.WMMButtonUp(var Message: TWMMButtonUp);
begin
  DoMouseUp(Message, mbMiddle);
end;

procedure TDateGrid.CMMouseLeave(var Message: TNotifyEvent);
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(self);
end;

procedure TDateGrid.CheckMenuPopup(const Pos: TSmallPoint);
var Control: TControl;
    PopupMenu: TPopupMenu;
begin
  if csDesigning in ComponentState then Exit;
  Control := Self;
  while Control <> nil do
    begin
    PopupMenu := TDateGrid(Control).GetPopupMenu;
    if (PopupMenu <> nil) and PopupMenu.AutoPopup then
      begin
      SendCancelMode(nil);
      PopupMenu.PopupComponent := Control;
      with ClientToScreen(SmallPointToPoint(Pos)) do PopupMenu.Popup(X, Y);
      Exit;
      end;
    Control := Control.Parent;
    end;
end;

procedure TDateGrid.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  MouseControl(Shift,X,Y,False);
  inherited;
end;

procedure TDateGrid.MouseControl(Shift: TShiftState; X, Y: Integer; Click: Boolean);
var tmpDate: TDate;
    tmpPoint: TPoint;
//    tmpRect: TRect;
begin
  FFirstDate.OnChange:=nil;
  FLastDate.OnChange:=nil;
  if Focused and (X>FLeftSpace) and (X<FLeftSpace+(HSpacing*7)-3) and (Y>FUpSpace-5) and (Y<FUpSpace+(6*VSpacing)) then
    begin
    if (ssLeft in Shift) and FLeftClick then
      begin
      if (FOldDay.x<>(X+3-FLeftSpace) div HSpacing) or (FOldDay.y<>IifInt(Y<FUpSpace,-1,(Y+1-FUpSpace) div VSpacing)) then
        begin
//        tmpPoint:=FWorkDate.CalendarPoint;
//        tmpRect:=Rect(tmpPoint.x*HSpacing-3+LeftSpace,tmpPoint.y*VSpacing-2+UpSpace,tmpPoint.x*HSpacing+17+LeftSpace,tmpPoint.y*VSpacing+16+UpSpace);
//        InvalidateRect(Handle,@tmpRect,True);
//        DrawCurrentDay;
        tmpDate:=TDate.Create;
        tmpDate.Assign(FWorkDate);
        FWorkDate.SetDateAsCalendarPoint((X+3-FLeftSpace) div HSpacing, IifInt(Y<FUpSpace,-1,(Y+1-FUpSpace) div VSpacing));
        if (tmpDate.Month<>FWorkDate.Month) or (tmpDate.Year<>FWorkDate.Year) then
          begin
//          tmpRect:=Rect(2,UpSpace-2,Width-2,Height-2);
//          InvalidateRect(Handle,@tmpRect,False);
          tmpPoint:=FWorkDate.CalendarPoint;
          tmpPoint:=Self.ClientToScreen(Point(tmpPoint.x*HSpacing-3+FLeftSpace,tmpPoint.y*VSpacing-1+FUpSpace));
          SetCursorPos(tmpPoint.x+10, tmpPoint.y+10);
          end;
        DrawWorkDate(clGray);
        tmpDate.Free;
        FOldDay:=FWorkDate.CalendarPoint;
        if goSingleDate in FGridOptions {FDateType=dtDateSingle} then
          begin
          FFirstDate.Assign(FWorkDate);
          FLastDate.Assign(FWorkDate);
          end
        else if Click then
          begin
          FFirstDate.Assign(FWorkDate);
          FLastDate.Assign(FWorkDate);
          end
        else if FWorkDate.Julian<FFirstDate.Julian then FFirstDate.Assign(FWorkDate) else FLastDate.Assign(FWorkDate);
//        tmpRect:=Rect(2,UpSpace-2,Width-2,Height-2);
//        InvalidateRect(Handle,@tmpRect,False);
        end;
      end;
    end;
  FFirstDate.OnChange:=CheckDateRange;
  FLastDate.OnChange:=CheckDateRange;
  Draw;
  inherited;
end;

procedure TDateGrid.PaintWindow(DC: HDC);
begin
  FCanvas.Lock;
  try
    FCanvas.Handle := DC;
    try
      FCanvas.Pen.Color:=clWhite;
      FCanvas.FillRect(Rect(0,0,150,150));
      Paint;
    finally
      FCanvas.Handle := 0;
    end;
  finally
    FCanvas.Unlock;
  end;
end;

procedure TDateGrid.Click;
begin
  inherited Click;
end;

procedure TDateGrid.KeyDown(var Key: Word; Shift: TShiftState);
var tmpDate: TDate;
begin
  if (Key <> 0) and not (ssAlt in Shift) then
    begin
    FFirstDate.OnChange:=nil;
    FLastDate.OnChange:=nil;
    if ((ssShift in Shift) or (ssCtrl in Shift)) and (goSingleDate in FGridOptions){(FDateType=dtDateSingle)} then
      begin
      Key:=0;
      inherited KeyDown(Key, Shift);
      Exit;
      end
    else
      begin
//      tmpPoint:=FWorkDate.CalendarPoint;
//      tmpRect:=Rect(tmpPoint.x*HSpacing-3+LeftSpace,tmpPoint.y*VSpacing-2+UpSpace,tmpPoint.x*HSpacing+17+LeftSpace,tmpPoint.y*VSpacing+16+UpSpace);
//      InvalidateRect(Handle,@tmpRect,False);
//      DrawCurrentDay;
      end;
    tmpDate:=TDate.Create;
    tmpDate.Assign(FWorkDate);
    if Key=VK_RETURN then
      begin
      Click;
      end
    else if (Key=VK_LEFT) or (Key=VK_RIGHT) then
      begin
      if Key=VK_RIGHT then FWorkDate.Increment else FWorkDate.Decrement;
      end
    else if (Key=VK_UP) or (Key=VK_DOWN) then
      begin
      if Key=VK_DOWN then FWorkDate.Add(7) else FWorkDate.Sub(7);
      end
    else if (Key=VK_PRIOR) or (Key=VK_NEXT) then
      begin
      if Key=VK_NEXT then FWorkDate.IncrementMonth(1) else FWorkDate.DecrementMonth(1);
      end
    else if (Key=VK_HOME) or (Key=VK_END) then
      begin
      if Key=VK_END then FWorkDate.Day:=FWorkDate.DaysInMonth else FWorkDate.Day:=1;
      end
    else if Key=VK_SPACE then
      begin
      if (ssShift in Shift) and not (goSingleDate in FGridOptions) then
        begin
        if FWorkDate.Julian<FFirstDate.Julian then FFirstDate.Assign(FWorkDate) else FLastDate.Assign(FWorkDate);
//        tmpRect:=Rect(2,UpSpace-2,Width-2,Height-2);
//        InvalidateRect(Handle,@tmpRect,False);
        end
      else
        begin
        if (FWorkDate.Month=FFirstDate.Month) and (FWorkDate.Year=FFirstDate.Year) then
          begin
//          if (FDateType=dtDateRange) and (FLastDate.SubDate(FFirstDate)>0) then Invalidate;
//          tmpPoint:=FFirstDate.CalendarPoint;
//          tmpRect:=Rect(tmpPoint.x*HSpacing-3+LeftSpace,tmpPoint.y*VSpacing-2+UpSpace,tmpPoint.x*HSpacing+17+LeftSpace,tmpPoint.y*VSpacing+16+UpSpace);
//          InvalidateRect(Handle,@tmpRect,True);
          FFirstDate.Assign(FWorkDate);
          FLastDate.Assign(FWorkDate);
//          tmpPoint:=FFirstDate.CalendarPoint;
//         tmpRect:=Rect(tmpPoint.x*HSpacing-3+LeftSpace,tmpPoint.y*VSpacing-2+UpSpace,tmpPoint.x*HSpacing+17+LeftSpace,tmpPoint.y*VSpacing+16+UpSpace);
//         InvalidateRect(Handle,@tmpRect,True);
          end
        else
          begin
//          if not (goSingleDate in FGridOptions){(FDateType=dtDateRange)} and (FLastDate.SubDate(FFirstDate)>0) then Invalidate;
          FFirstDate.Assign(FWorkDate);
          FLastDate.Assign(FWorkDate);
//          DrawEllipse(FFirstDate);
//          DrawCurrentDay;
          end;
        end;
      end;
    if not (ssShift in Shift) and not (ssCtrl in Shift) then
      begin
      if (tmpDate.Month<>FWorkDate.Month) or (tmpDate.Year<>FWorkDate.Year) then
        begin
//        tmpRect:=Rect(2,UpSpace-2,Width-2,Height-2);
//        InvalidateRect(Handle,@tmpRect,False);
        end
      else DrawWorkDate(clGray);
      end
    else if (ssShift in Shift) and not (goSingleDate in FGridOptions){(FDateType=dtDateRange)} and (Key<>16) then
      begin
      if FWorkDate.Julian<FFirstDate.Julian then FFirstDate.Assign(FWorkDate) else FLastDate.Assign(FWorkDate);
//      tmpRect:=Rect(2,UpSpace-2,Width-2,Height-2);
//      InvalidateRect(Handle,@tmpRect,False);
      end;
    tmpDate.Free;
    Key:=0;
    FFirstDate.OnChange:=CheckDateRange;
    FLastDate.OnChange:=CheckDateRange;
    Draw;
    end;
  inherited KeyDown(Key, Shift);
end;

procedure TDateGrid.KeyPress(var Key: Char);
begin
  if Key <> #0 then inherited KeyPress(Key);
  if (Key <> #0) and not (Char(Key) in [^V, ^X, ^C]) then
  begin
    Key := #0;
  end;
end;

procedure TDateGrid.DrawCurrentDay;
var tmpPoint: TPoint;
begin
  if (FWorkDate.Month=FCurrentDate.Month) and (FWorkDate.Year=FCurrentDate.Year) and (goViewToDay in FGridOptions) then
    begin
    tmpPoint:=FCurrentDate.CalendarPoint;
    Canvas.Draw(tmpPoint.x*HSpacing-3+FLeftSpace,tmpPoint.y*VSpacing-3+FUpSpace,TGraphic(FToDay));
    end;
end;

procedure TDateGrid.DrawEllipse(Date: TDate);
var tmpPoint: TPoint;
begin
  tmpPoint:=Date.CalendarPoint;
  Canvas.Draw(tmpPoint.x*HSpacing-1+FLeftSpace,tmpPoint.y*VSpacing-1+FUpSpace,TGraphic(FEllipse));
  Canvas.Font.Color:=clWhite;
  FCanvas.Brush.Style:=bsClear;
  Canvas.TextOut(tmpPoint.x*HSpacing+FLeftSpace+(Canvas.TextWidth('00')-Canvas.TextWidth(IntToStr(Date.Day))) div 2,tmpPoint.y*VSpacing+FUpSpace,IntToStr(Date.Day));
end;

procedure TDateGrid.DrawWorkDate(Color: TColor);
var tmpPoint: TPoint;
begin
  tmpPoint:=FWorkDate.CalendarPoint;
  FCanvas.Pen.Color:=Color;
  FCanvas.Pen.Style:=psDot;
  FCanvas.Pen.Mode:=pmCopy;
  FCanvas.Brush.Color:=clWhite;
  FCanvas.Brush.Style:=bsClear;
  FCanvas.Rectangle(tmpPoint.x*HSpacing-2+FLeftSpace,tmpPoint.y*VSpacing-2+FUpSpace,tmpPoint.x*HSpacing+17+FLeftSpace,tmpPoint.y*VSpacing+16+FUpSpace);
end;

procedure TDateGrid.DrawFrame;
begin
  Canvas.Brush.Color:=clBlack;
  Canvas.Pen.Color:=clBlack;
  Canvas.Pen.Style:=psSolid;
  if Ctl3D then
    begin
    Canvas.MoveTo(0,Height-1);
    Canvas.Pen.Color:=clGray;
    Canvas.LineTo(0,0);
    Canvas.LineTo(Width-1,0);
    Canvas.Pen.Color:=clWhite;
    Canvas.LineTo(Width-1,Height-1);
    Canvas.LineTo(1,Height-1);
    Canvas.MoveTo(1,Height-2);
    Canvas.Pen.Color:=clBlack;
    Canvas.LineTo(1,1);
    Canvas.LineTo(Width-2,1);
    Canvas.Pen.Color:=clSilver;
    Canvas.LineTo(Width-2,Height-2);
    Canvas.LineTo(1,Height-2);
    end
  else
    begin
    Canvas.FrameRect(Rect(0,0,Width,Height));
    Canvas.Brush.Color:=clWhite;
    Canvas.FrameRect(Rect(1,1,Width-1,Height-1));
    end;
end;

function TDateGrid.RepaintThisDate(CD, OD: TDate; i,j: Integer): Boolean;
begin
  Result:=True;
  Canvas.Brush.Style:=bsSolid;
  if IsDateFirst(OD, FOldWD) and not IsDateFirst(OD, FWorkDate) then
    begin
    Canvas.Brush.Color:=clWhite;
    Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-2+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+16+FUpSpace));
    Exit;
    end;
  if (CD.Day=OD.Day) then
    begin
    if IsDateInRange(CD,FFirstDate,FLastDate) then
      begin
      if IsDateInRange(OD,FOldFD,FOldLD) then
        begin
        if IsDateFirst(CD, FFirstDate) or IsDateLast(CD, FLastDate) then
          begin
          if ((IsDateFirst(CD, FFirstDate) and IsDateFirst(OD, FOldFD)) or (IsDateLast(CD, FLastDate) and IsDateLast(OD, FOldLD))) then
            begin
            if (FLastDate.SubDate(FFirstDate)>0) or ((FOldLD.SubDate(FOldFD)=0) and (FLastDate.SubDate(FFirstDate)=0)) then Result:=False else
              begin
              Canvas.Brush.Color:=clWhite;
              Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
              end;
            end
          else
            begin
            Canvas.Brush.Color:=clWhite;
            Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
            end;
          end
        else if IsDateFirst(OD, FOldFD) or IsDateLast(OD, FOldLD) then Result:=True else Result:=False;
        end
      else Result:=True;
      end
    else
      begin
      if IsDateInRange(OD,FOldFD,FOldLD) then
        begin
        Canvas.Brush.Color:=clWhite;
        Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
        end
      else Result:=False;
      end;
    end
  else
    begin
    Result:=True;
    if IsDateFirst(OD, FCurrentDate) then
      begin
      Canvas.Brush.Color:=clWhite;
      Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-2+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+FUpSpace));
      end;
    if IsDateInRange(CD,FFirstDate,FLastDate) then
      begin
      if not IsDateInRange(OD,FOldFD,FOldLD) then
        begin
        if IsDateFirst(CD, FFirstDate) or IsDateLast(CD, FLastDate) then
          begin
          Canvas.Brush.Color:=clWhite;
          Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
          end
        else
          begin
          Canvas.Brush.Color:=clNavy;
          Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
          end;
        end
      else
        begin
        Canvas.Brush.Color:=clWhite;
        Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
        end;
      end
    else
      begin
      Canvas.Brush.Color:=clWhite;
      Canvas.FillRect(Rect(j*HSpacing-3+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
      end;
    end;
end;

procedure TDateGrid.Draw;
var i,j: Integer;
    tmpDate, tmpOD: TDate;
begin
  tmpDate:=TDate.Create;
  tmpOD:=TDate.Create;
  if (FOldWD.Month<>FWorkDate.Month) or (FOldWD.Year<>FWorkDate.Year) then
    begin
    FLabel.Caption:=FWorkDate.StrMonth;
    FYearLabel.Caption:=IntToStr(FWorkDate.Year);
    FLabel.Left:=(Width div 2)-((FLabel.Width+FYearLabel.Width) div 2);
    FYearLabel.Left:=(Width div 2)-((FLabel.Width+FYearLabel.Width) div 2)+FLabel.Width;
    end;
  Canvas.Brush.Style:=bsSolid;
  Canvas.Brush.Color:=clWhite;
  Canvas.Font.Color:=clBlue;
  Canvas.Pen.Color:=clBlack;
  Canvas.Pen.Style:=psSolid;
  Canvas.Font.Style:=[];
  tmpDate.Assign(FWorkDate);
  tmpDate.Day:=1;
  tmpDate.Sub(tmpDate.DayOfWeek-1);
  tmpOD.Assign(FOldWD);
  tmpOD.Day:=1;
  tmpOD.Sub(tmpOD.DayOfWeek-1);
  for i:=0 to 5 do
    begin
    if (goViewWeekNumber in FGridOptions) and ((FOldWD.Month<>FWorkDate.Month) or (FOldWD.Year<>FWorkDate.Year)) then
      begin
      Canvas.Font.Color:=clBlack;
      Canvas.Brush.Color:=clWhite;
      Canvas.Brush.Style:=bsSolid;
      Canvas.Font.Style:=[fsBold];
      Canvas.TextOut(5,i*VSpacing+FUpSpace,IntToStr(tmpDate.WeekOfYear));
      Canvas.Font.Style:=[];
      end;
    for j:=0 to 6 do
      begin
      if RepaintThisDate(tmpDate, tmpOD, i, j) then
        begin
        if tmpDate.Month=FWorkDate.Month then Canvas.Font.Color:=clBlack else Canvas.Font.Color:=clGrayText;
        Canvas.Brush.Style:=bsClear;
        if IsDateInRange(tmpDate, FirstDate, LastDate) and not (goSingleDate in FGridOptions){(FDateType=dtDateRange)} and (FLastDate.SubDate(FFirstDate)>0) then
          begin
          if IsDateFirst(tmpDate, FFirstDate) or IsDateLast(tmpDate, FLastDate) then
            begin
            Canvas.Draw(j*HSpacing-1+FLeftSpace,i*VSpacing-1+FUpSpace,TGraphic(FEllipse));
            Canvas.Brush.Color:=clNavy;
            if IsDateFirst(tmpDate, FFirstDate) then Canvas.FillRect(Rect(j*HSpacing+10+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace))
            else Canvas.FillRect(Rect(j*HSpacing-2+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+10+FLeftSpace,i*VSpacing+15+FUpSpace));
            if tmpDate.Month=FWorkDate.Month then Canvas.Font.Color:=clWhite else Canvas.Font.Color:=clSilver;
            end
          else
            begin
            Canvas.Brush.Color:=clNavy;
            Canvas.FillRect(Rect(j*HSpacing-2+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
            if tmpDate.Month=FWorkDate.Month then Canvas.Font.Color:=clWhite else Canvas.Font.Color:=clSilver;
            end;
          end;
        Canvas.Brush.Style:=bsClear;
        Canvas.TextOut(j*HSpacing+FLeftSpace+(Canvas.TextWidth('00')-Canvas.TextWidth(IntToStr(tmpDate.Day))) div 2,i*VSpacing+FUpSpace,IntToStr(tmpDate.Day));
        end;
      tmpDate.Increment;
      tmpOD.Increment;
      end;
    end;
  if Focused then DrawWorkDate(clGray);
  if (FirstDate.Month=FWorkDate.Month) and (FirstDate.Year=FWorkDate.Year) then DrawEllipse(FFirstDate);
  DrawCurrentDay;
  tmpDate.Free;
  tmpOD.Free;
  Canvas.Brush.Color:=clBlack;
  Canvas.Pen.Color:=clBlack;
  Canvas.Pen.Style:=psSolid;
  FOldWD.Assign(FWorkDate);
  FOldFD.Assign(FFirstDate);
  FOldLD.Assign(FLastDate);
end;

function  TDateGrid.IsDateInRange(Date, FD, LD: TDate): Boolean;
begin
  Result:=False;
  if (Date.Julian>=FD.Julian) and (Date.Julian<=LD.Julian) then Result:=True;
end;

function  TDateGrid.IsDateFirst(Date, FD: TDate): Boolean;
begin
  Result:=False;
  if (Date.Year=FD.Year) and (Date.Month=FD.Month) and (Date.Day=FD.Day) then Result:=True;
end;

function  TDateGrid.IsDateLast(Date, LD: TDate): Boolean;
begin
  Result:=False;
  if (Date.Year=LD.Year) and (Date.Month=LD.Month) and (Date.Day=LD.Day) then Result:=True;
end;

procedure TDateGrid.Paint;
var i,j: Integer;
    tmpDate: TDate;
begin
  tmpDate:=TDate.Create;
  FLabel.Caption:=FWorkDate.StrMonth;
  FYearLabel.Caption:=IntToStr(FWorkDate.Year);
  FLabel.Left:=(Width div 2)-((FLabel.Width+FYearLabel.Width) div 2);
  FYearLabel.Left:=(Width div 2)-((FLabel.Width+FYearLabel.Width) div 2)+FLabel.Width;
  Canvas.Brush.Style:=bsSolid;
  Canvas.Brush.Color:=clNavy;
  Canvas.FillRect(Rect(0, 0, Width, 26));
  Canvas.Brush.Color:=clWhite;
  Canvas.FillRect(Rect(0, 26, Width, Height));
  Canvas.Font.Color:=clBlue;
  Canvas.Font.Style:=[fsBold];
  for i:=0 to 6 do case FLangType of
    ltCzech: Canvas.TextOut(i*HSpacing+FLeftSpace-2+(Canvas.TextWidth('00')-Canvas.TextWidth(DOWCZS[i+1])) div 2,32,DOWCZS[i+1]);
    else Canvas.TextOut(i*HSpacing+FLeftSpace-2+(Canvas.TextWidth('00')-Canvas.TextWidth(DOWENS[i+1])) div 2,32,DOWENS[i+1]);
    end;
  Canvas.Pen.Color:=clBlack;
  Canvas.Pen.Style:=psSolid;
  Canvas.Pen.Width:=1;
  Canvas.MoveTo(FLeftSpace-2,46);
  Canvas.LineTo(FLeftSpace-2+(HSpacing*7)-2,46);
  Canvas.Font.Style:=[];
  tmpDate.Assign(FWorkDate);
  tmpDate.Day:=1;
  tmpDate.Sub(tmpDate.DayOfWeek-1);
  if goViewWeekNumber in FGridOptions then
    begin
    Canvas.MoveTo(FLeftSpace-7,FUpSpace+2);
    Canvas.LineTo(FLeftSpace-7,FUpSpace+(VSpacing*6)-3);
    end;
  for i:=0 to 5 do
    begin
    if goViewWeekNumber in FGridOptions then
      begin
      Canvas.Font.Color:=clBlack;
      Canvas.Brush.Color:=clWhite;
      Canvas.Brush.Style:=bsSolid;
      Canvas.Font.Style:=[fsBold];
      Canvas.TextOut(5,i*VSpacing+FUpSpace,IntToStr(tmpDate.WeekOfYear));
      Canvas.Font.Style:=[];
      end;
    for j:=0 to 6 do
      begin
      if tmpDate.Month=FWorkDate.Month then Canvas.Font.Color:=clBlack else Canvas.Font.Color:=clGrayText;
      Canvas.Brush.Style:=bsClear;
      if IsDateInRange(tmpDate, FirstDate, LastDate) and not (goSingleDate in FGridOptions){(FDateType=dtDateRange)} and (FLastDate.SubDate(FFirstDate)>0) then
        begin
        if IsDateFirst(tmpDate, FFirstDate) or IsDateLast(tmpDate, FLastDate) then
          begin
          Canvas.Draw(j*HSpacing-1+FLeftSpace,i*VSpacing-1+FUpSpace,TGraphic(FEllipse));
          Canvas.Brush.Color:=clNavy;
          if IsDateFirst(tmpDate, FFirstDate) then Canvas.FillRect(Rect(j*HSpacing+10+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace))
          else Canvas.FillRect(Rect(j*HSpacing-2+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+10+FLeftSpace,i*VSpacing+15+FUpSpace));
          if tmpDate.Month=FWorkDate.Month then Canvas.Font.Color:=clWhite else Canvas.Font.Color:=clSilver;
          end
        else
          begin
          Canvas.Brush.Color:=clNavy;
          Canvas.FillRect(Rect(j*HSpacing-2+FLeftSpace,i*VSpacing-1+FUpSpace,j*HSpacing+(HSpacing-2)+FLeftSpace,i*VSpacing+15+FUpSpace));
          if tmpDate.Month=FWorkDate.Month then Canvas.Font.Color:=clWhite else Canvas.Font.Color:=clSilver;
          end;
        end;
      Canvas.TextOut(j*HSpacing+FLeftSpace+(Canvas.TextWidth('00')-Canvas.TextWidth(IntToStr(tmpDate.Day))) div 2,i*VSpacing+FUpSpace,IntToStr(tmpDate.Day));
      tmpDate.Increment;
      end;
    end;
  if Focused then DrawWorkDate(clGray);
  if (FirstDate.Month=FWorkDate.Month) and (FirstDate.Year=FWorkDate.Year) then DrawEllipse(FFirstDate);
  DrawCurrentDay;
  tmpDate.Free;
  DrawFrame;
end;

{DateButton}

constructor TDateButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] + [{csFramed,} csOpaque];
  FUpButton := CreateButton;
  FDownButton := CreateButton;
  UpGlyph := nil;
  DownGlyph := nil;
  Width := 20;
  Height := 18;
  FFocusedButton := FUpButton;
end;

function TDateButton.CreateButton: TDateSpeedButton;
begin
  Result := TDateSpeedButton.Create (Self);
  Result.OnClick := BtnClick;
  Result.OnMouseDown := BtnMouseDown;
  Result.Visible := True;
  Result.Enabled := True;
  Result.DateBtnState := [tbAllowTimer];
  Result.Height:=9;
  Result.Parent := Self;
end;

procedure TDateButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FFocusControl) then FFocusControl := nil;
end;

procedure TDateButton.AdjustSize (var W: Integer; var H: Integer);
begin
  if (FUpButton = nil) or (csLoading in ComponentState) then Exit;
  if W<14 then W:=14;
  FUpButton.SetBounds (0, 0, W, H div 2);
  FDownButton.SetBounds (0, FUpButton.Height - 1, W, H - FUpButton.Height + 1);
end;

procedure TDateButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  AdjustSize (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

procedure TDateButton.WMSize(var Message: TWMSize);
var W, H: Integer;
begin
  inherited;
  { check for minimum size }
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure TDateButton.WMSetFocus(var Message: TWMSetFocus);
begin
  FFocusedButton.DateBtnState := FFocusedButton.DateBtnState + [tbFocusRect];
  FFocusedButton.Invalidate;
end;

procedure TDateButton.WMKillFocus(var Message: TWMKillFocus);
begin
  FFocusedButton.DateBtnState := FFocusedButton.DateBtnState - [tbFocusRect];
  FFocusedButton.Invalidate;
end;

procedure TDateButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_UP:
      begin
        SetFocusBtn (FUpButton);
        FUpButton.Click;
      end;
    VK_DOWN:
      begin
        SetFocusBtn (FDownButton);
        FDownButton.Click;
      end;
    VK_SPACE:
      FFocusedButton.Click;
  end;
end;

procedure TDateButton.BtnMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    begin
    SetFocusBtn (TDateSpeedButton (Sender));
    if (FFocusControl <> nil) and FFocusControl.TabStop and
        FFocusControl.CanFocus and (GetFocus <> FFocusControl.Handle) then FFocusControl.SetFocus
    else if TabStop and (GetFocus <> Handle) and CanFocus then SetFocus;
    end;
end;

procedure TDateButton.BtnClick(Sender: TObject);
begin
  if Sender = FUpButton then
    begin
    if Assigned(FOnUpClick) then FOnUpClick(Self);
    end
  else if Assigned(FOnDownClick) then FOnDownClick(Self);
end;

procedure TDateButton.SetFocusBtn (Btn: TDateSpeedButton);
begin
  if TabStop and CanFocus and  (Btn <> FFocusedButton) then
    begin
    FFocusedButton.DateBtnState := FFocusedButton.DateBtnState - [tbFocusRect];
    FFocusedButton := Btn;
    if (GetFocus = Handle) then
      begin
      FFocusedButton.DateBtnState := FFocusedButton.DateBtnState + [tbFocusRect];
      Invalidate;
      end;
    end;
end;

procedure TDateButton.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TDateButton.Loaded;
var W, H: Integer;
begin
  inherited Loaded;
  Height:=18;
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height) then inherited SetBounds (Left, Top, W, H);
end;

function TDateButton.GetUpGlyph: TBitmap;
begin
  Result := FUpButton.Glyph;
end;

procedure TDateButton.SetUpGlyph(Value: TBitmap);
begin
  if Value <> nil then FUpButton.Glyph := Value else
    begin
    FUpButton.Glyph.Handle := LoadBitmap(HInstance, 'PLUS');
    FUpButton.NumGlyphs := 1;
    FUpButton.Invalidate;
    end;
end;

function TDateButton.GetUpNumGlyphs: TNumGlyphs;
begin
  Result := FUpButton.NumGlyphs;
end;

procedure TDateButton.SetUpNumGlyphs(Value: TNumGlyphs);
begin
  FUpButton.NumGlyphs := Value;
end;

function TDateButton.GetDownGlyph: TBitmap;
begin
  Result := FDownButton.Glyph;
end;

procedure TDateButton.SetDownGlyph(Value: TBitmap);
begin
  if Value <> nil then FDownButton.Glyph := Value else
    begin
    FDownButton.Glyph.Handle := LoadBitmap(HInstance, 'MINUS');
    FUpButton.NumGlyphs := 1;
    FDownButton.Invalidate;
    end;
end;

function TDateButton.GetDownNumGlyphs: TNumGlyphs;
begin
  Result:=FDownButton.NumGlyphs;
end;

procedure TDateButton.SetDownNumGlyphs(Value: TNumGlyphs);
begin
  FDownButton.NumGlyphs:=Value;
end;


{DateEdit}

constructor TCustomDateEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDateButton:=TDateSpeedButton.Create(self);
  FDateButton.Left:=1;
  FDateButton.Top:=1;
  FDateButton.Width:=16;
  FDateButton.Height:=16;
  FDateButton.Visible:=True;
  FDateButton.Parent:=self;
  FDateButton.Cursor:=crArrow;
  FDateButton.Flat:=False;
  FDateButton.Glyph.Handle := LoadBitmap(HInstance, 'DIARY');
  FDateButton.NumGlyphs := 1;
  FDateButton.Invalidate;
  FDateButton.OnClick:=DateButtonClick;
  FButton:=TDateButton.Create(Self);
  FButton.Width:=16;
  FButton.Height:=16;
  FButton.Visible:=True;
  FButton.Parent:=Self;
  FButton.FocusControl:=Self;
  FButton.OnUpClick:=UpClick;
  FButton.OnDownClick:=DownClick;
  FButton.FUpButton.Accelerate:=True;
  FButton.FDownButton.Accelerate:=True;
  FDateForm:=TForm.Create(self);
  FDateForm.OnDeactivate:=FormDeactivate;
  FDateForm.OnCreate:=FormCreate;
  FDateForm.FormStyle:=fsStayOnTop;
  FDateForm.BorderStyle:=bsNone;
  FDateGrid:=TDateGrid.Create(Self);
  FDateGrid.Parent:=FDateForm;
  FDateGrid.OnClick:=GridClick;
  FDateForm.Height:=FDateGrid.Height;
  FDateForm.Width:=FDateGrid.Width;
  ControlStyle:=ControlStyle-[csSetCaption];
  FCurrentDate:=TDate.Create;
  FWorkDate:=TDate.Create;
  FFirstDate:=TDate.Create;
  FLastDate:=TDate.Create;
  FFirstDate.OnChange:=ChangeDate;
  FLastDate.OnChange:=ChangeDate;
  FEditorEnabled := True;
  FLangType:=ltEnglish;
  MaxLength:=10;
//  Text:=FWorkDate.DateToString;
  FPosition:=0;
  FMask:='NN'+DateSeparator+'NN'+DateSeparator+'NNNN';
  FDateIncrement:=diDay;
  FFormVisible:=False;
end;

destructor TCustomDateEdit.Destroy;
begin
  FCurrentDate.Free;
  FWorkDate.Free;
  FFirstDate.Free;
  FLastDate.Free;
  inherited Destroy;
end;

procedure TCustomDateEdit.Loaded;
begin
  inherited Loaded;
  case FDateType of
    dtDateSingle: Text:=FFirstDate.DateToString;
    dtDateRange: Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
    end;
end;

procedure TCustomDateEdit.DoChange;
begin
end;

procedure TCustomDateEdit.SetDateType(Value: TDateType);
begin
  if Value=FDateType then Exit;
  FDateType:=Value;
  case FDateType of
    dtDateSingle: begin
                  FMask:='NN'+DateSeparator+'NN'+DateSeparator+'NNNN';
                  MaxLength:=10;
                  Text:=FFirstDate.DateToString;
                  end;
    dtDateRange: begin
                 FMask:='NN'+DateSeparator+'NN'+DateSeparator+'NNNN'+'-'+'NN'+DateSeparator+'NN'+DateSeparator+'NNNN';
                 MaxLength:=21;
                 Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
                 end;
    end;
end;

procedure TCustomDateEdit.SetFirstDate(Value: TDate);
begin
  if Value=FFirstDate then Exit;
  FFirstDate.Assign(Value);
  DoChange;
end;

procedure TCustomDateEdit.SetLastDate(Value: TDate);
begin
  if Value=FLastDate then Exit;
  FLastDate.Assign(Value);
  DoChange;
end;

procedure TCustomDateEdit.SetWorkDate(Value: TDate);
begin
  if Value=FWorkDate then Exit;
  FWorkDate.Assign(Value);
end;

procedure TCustomDateEdit.SetLangType(Value: TLangType);
begin
  if Value=FLangType then Exit;
  FLangType:=Value;
  FWorkDate.LangType:=Value;
  FFirstDate.LangType:=Value;
  FLastDate.LangType:=Value;
  FCurrentDate.LangType:=Value;
end;

procedure TCustomDateEdit.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TCustomDateEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
//  if Key = VK_UP then UpClick (Self)
//  else if Key = VK_DOWN then DownClick (Self);
  if (Key <> 0) and not (ssAlt in Shift) then
  begin
    if (Key = VK_LEFT) or(Key = VK_RIGHT) then
      begin
      LeftRightKeys(Key, Shift);
      Key:=0;
      Exit;
      end
    else if (Key=VK_UP) or (Key=VK_DOWN) then
      begin
      if Key=VK_UP then UpClick(self) else DownClick(self);
      Key := 0;
      Exit;
      end
    else if (Key = VK_HOME) or(Key = VK_END) then
      begin
      HomeEndKeys(Key, Shift);
      Key := 0;
      Exit;
      end
    else if ((Key = VK_DELETE) and ([ssShift, ssCtrl] * Shift = [])) or (Key = VK_BACK) then
      begin
      DeleteKeys(Key);
      Key := 0;
      Exit;
      end;
//    CheckCursor;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TCustomDateEdit.KeyPress(var Key: Char);
begin
  if not IsValidChar(Key) then
  begin
    Key := #0;
    MessageBeep(0)
  end;
  if Key <> #0 then inherited KeyPress(Key);
  if (Key <> #0) and not (Char(Key) in [^V, ^X, ^C]) then
  begin
    CharKeys(Key);
    Key := #0;
  end;
end;

function TCustomDateEdit.IsValidChar(Key: Char): Boolean;
begin
  Result := (Key in [DateSeparator, DecimalSeparator, '-', '0'..'9']) or ((Key < #32) and (Key <> Chr(VK_RETURN)));
  if not FEditorEnabled and Result and ((Key >= #32) or (Key = Char(VK_BACK)) or (Key = Char(VK_DELETE))) then Result := False;
end;

procedure TCustomDateEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
{  Params.Style := Params.Style and not WS_BORDER;  }
  Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TCustomDateEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

procedure TCustomDateEdit.CheckCursor;
var SelStart, SelStop: Integer;
begin
  if not HandleAllocated then  Exit;
  GetSel(SelStart, SelStop);
end;

function TCustomDateEdit.CharKeys(var CharCode: Char): Boolean;
var SelStart, i, tmpPos: Integer;
begin
  Result := False;
  if Word(CharCode) = VK_ESCAPE then
  begin
    Exit;
  end;
  if (Word(CharCode) = VK_BACK) then Exit;
  if (Word(CharCode) = VK_RETURN) then Exit;
  if (FPosition>=MaxLength) and (GetSelLength<2) then Exit;
  if CharCode in [DateSeparator, DecimalSeparator, '-'] then
    begin
    tmpPos:=0;
    for i:=1 to FPosition+1 do if FMask[i] in [DateSeparator, '-'] then tmpPos:=i;
    for i:=FPosition+1 to MaxLength+1 do
      begin
      if (FMask[i] in [DateSeparator, '-']) or (i>MaxLength) then
        begin
        Text:=Copy(Text,1,tmpPos)+CharCount('0',i-FPosition-1)+Copy(Text,tmpPos+1,FPosition-tmpPos)+Copy(Text,i,MaxLength-i+1);
        Break;
        end;
      end;
    end;
  if CharCode in [DateSeparator, DecimalSeparator] then
    begin
    for i:=FPosition to MaxLength+1 do if (FMask[i+1] in [DateSeparator, '-']) or (i>MaxLength) then
      begin
      FPosition:=i+1;
      Break;
      end;
    CheckValidDate;
    SetSel(FPosition,FPosition+1);
    Exit;
    end;
  if CharCode='-' then
    begin
    for i:=FPosition to MaxLength do if FMask[i+1]='-' then
      begin
      FPosition:=i+1;
      Break;
      end;
    CheckValidDate;
    SetSel(FPosition,FPosition+1);
    Exit;
    end;
  SelStart:=GetSelStart;
  if GetSelLength>0 then DeleteKeys(VK_DELETE);
  FPosition:=SelStart;
  SetSel(FPosition,FPosition+1);
  if GetSelLength>0 then SendMessage(Handle,WM_CLEAR,0,0);
  Text:=LeftStr(Text,FPosition)+CharCode+Right(Text,Length(Text)-FPosition);
  Inc(FPosition);
  if FMask[FPosition+1]<>'N' then Inc(FPosition);
  CheckValidDate;
  SendMessage(Handle,WM_SETTEXT,0,LongInt(PChar(Text)));
  SetSelStart(FPosition);
  SetSelLength(1);
end;

procedure TCustomDateEdit.CheckValidDate;
begin
  case FPosition of
    0,1,2: begin
           end;
    3,4,5: begin
           if StrToInteger(Copy(Text,1,2))=0 then Text:='01'+Copy(Text,3,MaxLength-2) else
             begin
             if StrToInteger(Copy(Text,1,2))>31 then Text:='31'+Copy(Text,3,MaxLength-2);
             end;
           end;
    6,7,8,9,10: begin
                if StrToInteger(Copy(Text,4,2))=0 then Text:=Copy(Text,1,3)+'01'+Copy(Text,6,MaxLength-5) else
                  begin
                  if StrToInteger(Copy(Text,4,2))>12 then Text:=Copy(Text,1,3)+'12'+Copy(Text,6,MaxLength-5);
                  end;
                end;
    11,12,13: begin
              if StrToInteger(Copy(Text,7,4))=0 then Text:=Copy(Text,1,6)+'0000'+Copy(Text,11,MaxLength-10);
              FFirstDate.StringToDate(LeftStr(Text,10));
              if FDateType=dtDateSingle then FLastDate.Assign(FFirstDate);
              DoChange;
              end;
    14,15,16: begin
              if StrToInteger(Copy(Text,12,2))=0 then Text:=Copy(Text,1,11)+'01'+Copy(Text,14,MaxLength-13) else
                begin
                if StrToInteger(Copy(Text,12,2))>31 then Text:=Copy(Text,1,11)+'31'+Copy(Text,14,MaxLength-13);
                end;
              end;
    17,18,19,20,21: begin
                    if StrToInteger(Copy(Text,15,2))=0 then Text:=Copy(Text,1,14)+'01'+Copy(Text,17,MaxLength-16) else
                      begin
                      if StrToInteger(Copy(Text,15,2))>12 then Text:=Copy(Text,1,14)+'12'+Copy(Text,17,MaxLength-16);
                      end;
                    end;
    22: begin
        if StrToInteger(Copy(Text,18,4))=0 then Text:=Copy(Text,1,17)+'0000';
        FLastDate.StringToDate(Right(Text,10));
        DoChange;
        end;
    end;
end;

procedure TCustomDateEdit.LeftRightKeys(CharCode: Word; Shift: TShiftState);
var SelStart, SelStop : Integer;
begin
  if (ssCtrl in Shift) then Exit;
  GetSel(SelStart, SelStop);
  if (ssShift in Shift) then
    begin
    if (CharCode = VK_RIGHT) then
      begin
      if (SelStop-SelStart=1) or (FPosition=SelStop) then
        begin
        if SelStop<MaxLength then Inc(SelStop);
        FPosition:=SelStop;
        end
      else
        begin
        Inc(SelStart);
        FPosition:=SelStart;
        end;
      SetSelStart(SelStart);
      SetSelLength(SelStop-SelStart);
      end
    else  {if (CharCode = VK_LEFT) then}
      begin
      if (SelStop-SelStart=1) or (FPosition=SelStart) then
        begin
        if SelStart>0 then Dec(SelStart);
        FPosition:=SelStart;
        end
      else
        begin
        Dec(SelStop);
        FPosition:=SelStop;
        end;
      SetSelStart(SelStart);
      SetSelLength(SelStop-SelStart);
      end;
    end
  else
    begin
    if (CharCode = VK_RIGHT) then
      begin
      if FPosition<MaxLength then Inc(FPosition);
      if (FMask[FPosition+1]<>'N') and (FPosition<MaxLength) then Inc(FPosition);
      SetSelStart(FPosition);
      SetSelLength(1);
      end
    else
      begin
      if FPosition>0 then Dec(FPosition);
      if FMask[FPosition+1]<>'N' then Dec(FPosition);
      SetSelStart(FPosition);
      SetSelLength(1);
      end;
    end;
end;

procedure TCustomDateEdit.HomeEndKeys(CharCode: Word; Shift: TShiftState);
var
  SelStart, SelStop, SelLength : Integer;
begin
  SelStart:=GetSelStart;
  SelLength:=GetSelLength;
  SelStop:=SelStart+SelLength;
  if (CharCode = VK_HOME) then
    begin
    if (ssShift in Shift) then
      begin
      if (SelStart <> FPosition) and (SelStop <> (SelStart + 1)) then SelStop := SelStart + 1;
      SetSelStart(0);
      SetSelLength(SelStop);
      FPosition:=SelStop;
      CheckCursor;
      end
    else
      begin
      SetSelStart(0);
      SetSelLength(1);
      FPosition:=0;
      end;
    end
  else
    begin
    if (ssShift in Shift) then
      begin
//      if (SelStop <> FPosition) and (SelStop <> (SelStart + 1)) then SelStart := SelStop - 1;
      SetSelStart(FPosition);
      SetSelLength(Length(Text)-FPosition);
      FPosition:=Length(Text);
      CheckCursor;
      end
    else
      begin
      SetSelStart(Length(Text));
      FPosition:=Length(Text);
      end;
    end;
end;

procedure TCustomDateEdit.DeleteKeys(CharCode: Word);
var
  SelStart, SelStop : Integer;
  Str: string;
  i: Integer;
begin
  GetSel(SelStart, SelStop);
  if CharCode=VK_BACK then
    begin
    if SelStop-SelStart=0 then if FPosition>0 then
      begin
      SelStart:=FPosition-1;
      SelStop:=FPosition;
      end;
    Str:=Text;
    for i:=SelStart to SelStop-1 do if FMask[i+1]='N' then Str[i+1]:=' ';
    SendMessage(Handle,WM_SETTEXT,0,LongInt(PChar(Str)));
    if SelStart>0 then FPosition:=SelStart-1;
    if FMask[FPosition+1]<>'N' then Dec(FPosition);
    SetSel(FPosition,FPosition+1);
    end
  else
    begin
    if SelStop-SelStart=0 then if FPosition<MaxLength then
      begin
      SelStart:=FPosition;
      SelStop:=FPosition+1;
      end;
    Str:=Text;
    for i:=SelStart to SelStop-1 do if FMask[i+1]='N' then Str[i+1]:=' ';
    SendMessage(Handle,WM_SETTEXT,0,LongInt(PChar(Str)));
    if SelStop<MaxLength then FPosition:=SelStop;
    if FMask[FPosition+1]<>'N' then Inc(FPosition);
    SetSel(FPosition,FPosition+1);
    end;
end;

procedure TCustomDateEdit.GetSel(var SelStart: Integer; var SelStop: Integer);
begin
  SendMessage(Handle, EM_GETSEL, Integer(@SelStart), Integer(@SelStop));
end;

procedure TCustomDateEdit.SetSel(SelStart: Integer; SelStop: Integer);
begin
  SendMessage(Handle, EM_SETSEL, SelStart, SelStop);
end;

procedure TCustomDateEdit.SetEditRect;
var Loc: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
  Loc.Bottom:=ClientHeight+1;  {+1 is workaround for windows paint bug}
  Loc.Right:=ClientWidth-FButton.Width-FDateButton.Width-2;
  Loc.Top := 0;
  Loc.Left := 0;
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));  {debug}
end;

procedure TCustomDateEdit.ChangeDate(Sender: TObject);
begin
  case FDateType of
    dtDateSingle: Text:=FFirstDate.DateToString;
    dtDateRange: Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
    end;
end;

procedure TCustomDateEdit.CheckDate(Sender: TObject);
var tmpDate: TDate;
begin
  FFirstDate.OnChange:=nil;
  FLastDate.OnChange:=nil;
  case FDateType of
    dtDateSingle: begin
                  FFirstDate.StringToDate(LeftStr(Text,10));
                  if FDateType=dtDateSingle then FLastDate.Assign(FFirstDate);
                  end;
    dtDateRange: begin
                 FFirstDate.StringToDate(LeftStr(Text,10));
                 FLastDate.StringToDate(Right(Text,10));
                 end;
    end;
  if FFirstDate.Julian>FLastDate.Julian then
    begin
    tmpDate:=TDate.Create;
    tmpDate.Assign(FFirstDate);
    FFirstDate.Assign(FLastDate);
    FLastDate.Assign(tmpDate);
    tmpDate.Free;
    end;
  case FDateType of
    dtDateSingle: Text:=FFirstDate.DateToString;
    dtDateRange: Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
    end;
  FFirstDate.OnChange:=ChangeDate;
  FLastDate.OnChange:=ChangeDate;
end;

procedure TCustomDateEdit.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  if AutoSelect then SetSel(0,MaxLength) else SetSel(0,1);
end;

procedure TCustomDateEdit.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  CheckDate(nil);
end;

procedure TCustomDateEdit.WMLButtonDown(var Message: TWMLButtonDown);
begin
  inherited;
  FBtnDownX := Message.XPos;
end;

procedure TCustomDateEdit.WMLButtonUp(var Message: TWMLButtonUp);
var SelStart, SelStop: Integer;
begin
  inherited;
  GetSel(SelStart, SelStop);
  FPosition:=SelStart;
  if (SelStart<>SelStop) and (Message.XPos>FBtnDownX) then FPosition:=SelStop;
  if SelStart=SelStop then
    begin
    if FMask[FPosition+1]<>'N' then
      begin
      Inc(FPosition);
      SetSelStart(FPosition);
      end;
    SetSelLength(1);
    end;
end;

procedure TCustomDateEdit.WMCut(var Message: TMessage);
begin
  CopyToClipboard;
  DeleteKeys(VK_DELETE);
end;

procedure TCustomDateEdit.WMPaste(var Message: TMessage);
var Value: string;
    Str: string;
    SelStart, SelStop, i: Integer;
begin
  if ReadOnly then Exit;
  Clipboard.Open;
  Value:=Clipboard.AsText;
  Clipboard.Close;
  GetSel(SelStart, SelStop);
  Str:=Text;
  DeleteKeys(VK_DELETE);
  for i:=1 to Length(Value) do
    begin
    if SelStart+i>MaxLength then Exit;
    if FMask[SelStart+i]='N' then
      begin
      if Value[i] in ['0'..'9'] then Str[SelStart+i]:=Value[i] else Str[SelStart+i]:=' ';
      end;
    SelStop:=SelStart+i;
    end;
  Text:=Str;
//  SendMessage(Handle,WM_SETTEXT,0,LongInt(PChar(Str)));
  FPosition:=SelStop;
  if FMask[FPosition+1]<>'N' then Inc(FPosition);
  SetSel(FPosition,FPosition+1);
  CheckDate(nil);
  FWorkDate.Assign(FFirstDate);
  DoChange;
end;

procedure TCustomDateEdit.WMSize(var Message: TWMSize);
var MinHeight: Integer;
begin
  inherited;
  MinHeight:=GetMinHeight;
    { text edit bug: if size to less than minheight, then edit ctrl does
      not display the text }
  if Height<MinHeight then Height:=MinHeight else if FButton <> nil then
    begin
    if NewStyleControls and Ctl3D then
      FButton.SetBounds(Width-FButton.Width-FDateButton.Width-5, 0, FButton.Width, Height-4)
    else FButton.SetBounds (Width-FButton.Width, 1, FButton.Width, Height-2);
    FDateButton.SetBounds(Width-FDateButton.Width-5, 0, FDateButton.Width, Height-4);
    SetEditRect;
    end;
end;

procedure TCustomDateEdit.CMMouseEnter (var Msg: TMessage);
begin
  if Assigned(FMouseEnter) then FMouseEnter(Self);
end;

procedure TCustomDateEdit.CMMouseLeave (var Msg: TMessage);
begin
  if Assigned(FMouseLeave) then FMouseLeave(Self);
end;

function TCustomDateEdit.GetMinHeight: Integer;
var DC: HDC;
    SaveFont: HFont;
    i: Integer;
    SysMetrics, Metrics: TTextMetric;
begin
  DC:=GetDC(0);
  GetTextMetrics(DC, SysMetrics);
  SaveFont:=SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  i:=SysMetrics.tmHeight;
  if i>Metrics.tmHeight then i:=Metrics.tmHeight;
  Result:=Metrics.tmHeight+i div 4+GetSystemMetrics(SM_CYBORDER)*4+2;
end;

procedure TCustomDateEdit.UpClick(Senter: TObject);
begin
  if not ReadOnly then
    begin
    case FDateIncrement of
      diDay: if FDateType=dtDateSingle then FFirstDate.Add(FButton.FUpButton.FIncrement) else FLastDate.Add(FButton.FUpButton.FIncrement);
      diMonth: if FDateType=dtDateSingle then FFirstDate.IncrementMonth(FButton.FUpButton.FIncrement) else FLastDate.IncrementMonth(FButton.FUpButton.FIncrement);
      diYear: if FDateType=dtDateSingle then FFirstDate.Year:=FWorkDate.Year+(FButton.FUpButton.FIncrement) else FLastDate.Year:=FLastDate.Year+(FButton.FUpButton.FIncrement);
      end;
    if FDateType=dtDateSingle then Text:=FFirstDate.DateToString else Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
    SetSel(0,MaxLength);
    FPosition:=MaxLength;
    FWorkDate.Assign(FFirstDate);
    DoChange;
    end;
end;

procedure TCustomDateEdit.DownClick(Senter: TObject);
begin
  if not ReadOnly then
    begin
    case FDateIncrement of
      diDay: if FDateType=dtDateSingle then FFirstDate.Sub(FButton.FDownButton.FIncrement) else FLastDate.Sub(FButton.FDownButton.FIncrement);
      diMonth: if FDateType=dtDateSingle then FFirstDate.DecrementMonth(FButton.FDownButton.FIncrement) else FLastDate.DecrementMonth(FButton.FDownButton.FIncrement);
      diYear: if FDateType=dtDateSingle then FFirstDate.Year:=FWorkDate.Year-FButton.FDownButton.FIncrement else FLastDate.Year:=FLastDate.Year-FButton.FDownButton.FIncrement;
      end;
    if FDateType=dtDateSingle then Text:=FFirstDate.DateToString else Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
    SetSel(0,MaxLength);
    FPosition:=MaxLength;
    FWorkDate.Assign(FFirstDate);
    DoChange;
    end;
end;

procedure TCustomDateEdit.DateButtonClick(Senter: TObject);
var tmpPoint: TPoint;
begin
  if not Focused then SetFocus;
  if FFormVisible then FDateForm.Hide else
    begin
    tmpPoint:=Self.ClientToScreen(Point(Width,Height));
    if tmpPoint.X-FDateForm.Width-2<0 then FDateForm.Left:=0 else FDateForm.Left:=tmpPoint.X-FDateForm.Width-2;
    if tmpPoint.Y+FDateForm.Height>Screen.Height then FDateForm.Top:=tmpPoint.y-Height-FDateForm.Height-2 else FDateForm.Top:=tmpPoint.Y;
    if FDateType=dtDateSingle then FDateGrid.Options:=FDateGrid.Options+[goSingleDate] else FDateGrid.Options:=FDateGrid.Options-[goSingleDate];
    FDateGrid.WorkDate.Assign(FFirstDate);
    FDateGrid.FirstDate.Assign(FFirstDate);
    FDateGrid.LastDate.Assign(FLastDate);
    FDateGrid.LangType:=FLangType;
    FDateGrid.Invalidate;
    FDateForm.Show;
    DoChange;
    end;
  FFormVisible:=not FFormVisible;
end;

procedure TCustomDateEdit.SetDateIncrement(Value: TDateIncrement);
begin
  if Value=FDateIncrement then Exit;
  FDateIncrement:=Value;
end;

procedure TCustomDateEdit.FormDeactivate(Sender: TObject);
begin
  FDateForm.Hide;
  FFirstDate.Assign(FDateGrid.FirstDate);
  FLastDate.Assign(FDateGrid.LastDate);
  FWorkDate.Assign(FDateGrid.WorkDate);
  case FDateType of
    dtDateSingle: Text:=FFirstDate.DateToString;
    dtDateRange: Text:=FFirstDate.DateToString+'-'+FLastDate.DateToString;
    end;
  CheckDate(nil);
  SetFocus;
end;

procedure TCustomDateEdit.FormCreate(Sender: TObject);
begin
  FDateGrid:=TDateGrid.Create(self);
  FDateGrid.Parent:=Self;
end;

procedure TCustomDateEdit.GridClick(Sender: TObject);
begin
  FormDeactivate(Sender);
  FFormVisible:=False;
end;

{DBDateEdit}

constructor TDBDateEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TDBDateEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TDBDateEdit.Loaded;
begin
  inherited Loaded;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TDBDateEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation=opRemove) and (FDataLink<>nil) and (AComponent=DataSource) then DataSource:=nil;
end;

procedure TDBDateEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key=VK_DELETE) or ((Key=VK_INSERT) and (ssShift in Shift)) then FDataLink.Edit;
  inherited KeyDown(Key, Shift);
end;

procedure TDBDateEdit.KeyPress(var Key: Char);
begin
  case Key of
    ^H, ^V, ^X, #32..#255:
      FDataLink.Edit;
    #27: begin
         FDataLink.Reset;
         SelectAll;
         Key := #0;
         end;
    end;
  inherited KeyPress(Key);
end;

function TDBDateEdit.EditCanModify: Boolean;
begin
  Result:=FDataLink.Edit;
end;

procedure TDBDateEdit.Reset;
begin
  FDataLink.Reset;
  SelectAll;
end;

procedure TDBDateEdit.SetFocused(Value: Boolean);
begin
  if FFocused<>Value then
    begin
    FFocused:=Value;
    Invalidate;
    FDataLink.Reset;
    end;
end;

procedure TDBDateEdit.DoChange;
begin
  FDataLink.Edit;
end;

procedure TDBDateEdit.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

function TDBDateEdit.GetDataSource: TDataSource;
begin
  Result:=FDataLink.DataSource;
end;

procedure TDBDateEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource:=Value;
  if Value<>nil then Value.FreeNotification(Self);
end;

function TDBDateEdit.GetDataField: string;
begin
  Result:=FDataLink.FieldName;
end;

procedure TDBDateEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName:=Value;
end;

function TDBDateEdit.GetReadOnly: Boolean;
begin
  Result:=FDataLink.ReadOnly;
end;

procedure TDBDateEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly:=Value;
end;

function TDBDateEdit.GetField: TField;
begin
  Result:=FDataLink.Field;
end;

procedure TDBDateEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field<>nil then
    begin
    if FAlignment<>FDataLink.Field.Alignment then
      begin
      FirstDate.Value:=1;
      Text:=FirstDate.DateToString;
      FAlignment:=FDataLink.Field.Alignment;
      end;
    if FFocused and FDataLink.CanModify then
      begin
      FirstDate.Value:=FDataLink.Field.AsDateTime;
      Text:=FirstDate.DateToString;
      end
    else
      begin
      FirstDate.Value:=FDataLink.Field.AsDateTime;
      Text:=FirstDate.DateToString;
      if FDataLink.Editing and FDataLink.CanModify then Modified:=True;
      end;
    end
  else
    begin
    FAlignment:=taLeftJustify;
    FirstDate.Value:=1;
    Text:=FirstDate.DateToString;
    end;
end;

procedure TDBDateEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly:=not FDataLink.Editing;
end;

procedure TDBDateEdit.UpdateData(Sender: TObject);
begin
  FDataLink.Field.Text:=Text;
end;

procedure TDBDateEdit.WMPaste(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TDBDateEdit.WMCut(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TDBDateEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;
  if SysLocale.FarEast and FDataLink.CanModify then inherited ReadOnly:=False;
end;

procedure TDBDateEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
  CheckCursor;
  DoExit;
end;

procedure TDBDateEdit.CMGetDataLink(var Message: TMessage);
begin
  message.Result := LongInt(fDataLink);
end;

{DateSpeedButton}

constructor TDateSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAccelerate:=False;
end;

destructor TDateSpeedButton.Destroy;
begin
  if FRepeatTimer <> nil then FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TDateSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  FCount:=0;
  FIncrement:=1;
  FPosInArray:=0;
  if tbAllowTimer in FDateBtnState then
    begin
    if FRepeatTimer = nil then FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := InitRepeatPause;
    FRepeatTimer.Enabled  := True;
    end;
end;

procedure TDateSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then FRepeatTimer.Enabled  := False;
end;

procedure TDateSpeedButton.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  if (FState = bsDown) and MouseCapture then
    begin
    if FAccelerate then
      begin
      Inc(FCount);
      if FCount>20 then
        begin
        if FPosInArray<10 then Inc(FPosInArray);
        FIncrement:=AccelerateArray[FPosInArray];
        FCount:=0;
        end;
      end;
    try
      Click;
    except
      FRepeatTimer.Enabled := False;
      raise;
      end;
    end;
end;

procedure TDateSpeedButton.Paint;
var R: TRect;
begin
  inherited Paint;
  if tbFocusRect in FDateBtnState then
    begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
    end;
end;

{Field property editor}

function TFieldProperty.GetAttributes: TPropertyAttributes;
begin
  Result:=[paValueList];
end;

procedure TFieldProperty.GetValues(Proc: TGetStrProc);
var i: Integer;
    tmpDBDateEdit: TDBDateEdit;
begin
    tmpDBDateEdit:=GetComponent(0) as TDBDateEdit;
    if tmpDBDateEdit.DataSource.DataSet<>nil then if tmpDBDateEdit.DataSource.DataSet.Active then
      for i:=0 to tmpDBDateEdit.DataSource.DataSet.FieldCount-1 do
        if (tmpDBDateEdit.DataSource.DataSet.Fields[i].DataType=ftDate) or
           (tmpDBDateEdit.DataSource.DataSet.Fields[i].DataType=ftDateTime) then Proc(tmpDBDateEdit.DataSource.DataSet.Fields[i].FieldName);
end;

end.
