unit MyProc;

interface

uses SysUtils, Math, Classes, Windows;

const
  DOWCZ: array[1..7] of string = ('pondl', 'ter', 'steda', 'tvrtek', 'ptek', 'sobota', 'nedle');
  DOWCZS: array[1..7] of string = ('Po','t','St','t','P','So','Ne');
  MonthCZ: array[1..12] of string = ('leden','nor','bezen','duben','kvten','erven','ervenec',
                                      'srpen','z','jen','listopad','prosinec');
  DOWEN: array[1..7] of string = ('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday');
  DOWENS: array[1..7] of string = ('Mo','Tu','We','Th','Fr','Sa','Su');
  MonthEN: array[1..12] of string = ('january','february','march','april','may','june','july',
                                      'august','september','october','november','december');

type

TLangType = (ltEnglish, ltCzech);
TDate = class(TPersistent)
  private
    FCentury: Boolean;
    FDay: Byte;
    FLangType: TLangType;
    FMonth: Byte;
    FYear: Word;
    FOnChange: TNotifyEvent;
    procedure ChangeDate;
    function  GetValue: TDateTime;
    procedure SetDay(Value: Byte);
    procedure SetLangType(Value: TLangType);
    procedure SetMonth(Value: Byte);
    procedure SetValue(Value: TDateTime);
    procedure SetYear(Value: Word);
    function GetLeapYear: Boolean;
    function DIM(Value: Byte; ValueY: Word): Byte;
    function LY(Value: Word): Word;
  protected
    procedure CheckValidDate;
  public
    procedure Add(Count: Integer);
    procedure Assign(Source: TPersistent); override;
    function  CalendarPoint: TPoint;
    constructor Create;
    function  DateToString: string;
    function  DayOfWeek: Byte;
    function  DayOfYear: Word;
    function  DaysInMonth: Byte;
    procedure Decrement;
    procedure DecrementMonth(Count: Integer);
    procedure Increment;
    procedure IncrementMonth(Count: Integer);
    function  Julian: Integer;
    procedure JulianToDate(JDate: Integer);
    procedure SetDateAsCalendarPoint(x, y: Integer);
    function  StrDayOfWeek: string;
    procedure StringToDate(Value: string);
    function  StrMonth: string;
    procedure Sub(Count: Integer);
    function  SubDate(XDate: TDate): Integer;
    procedure SysDate;
    function  WeekOfYear: Word;
  published
    property Century: Boolean read FCentury write FCentury;
    property Day: Byte read FDay write SetDay;
    property LangType: TLangType read FLangType write SetLangType;
    property LeapYear: Boolean read GetLeapYear;
    property Month: Byte read FMonth write SetMonth;
    property Value: TDateTime read GetValue write SetValue;
    property Year: Word read FYear write SetYear;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;


function  PadC(Text: string; Count: Word; Character: Char): string;
function  PadL(Text: string; Count: Word; Character: Char): string;
function  PadR(Text: string; Count: Word; Character: Char): string;
function  LeftStr(Text: string; Count: Word): string;
function  Right(Text: string; Count: Word): string;
function  FindChar(Text: String; Symbol: Char; N: Byte): Integer;
function  CountChar(Text: String; Ch: Char): Integer;
function  CharCount(Znak: Char; Count: Integer): string;
function  MinusStr(Text1, Text2: string): string;
function  IifInt(Podminka: Boolean; Value1, Value2: Integer): Integer;
function  IifStr(Podminka: Boolean; Value1, Value2: string): string;
function  CodeString(Text, Code: string): string;
function  DecodeString(Text, Code: string): string;
function  SqrX(Value: Extended; Count: Integer): Extended;
function  Question(Text: string): Boolean;
function  StrToInteger(Text: string): Integer;
function  AbsInt(Value: Integer): Integer;
function  InsertStrRight(SourceText, InsertText: string; Position: Integer): string;
function  InsertStrLeft(SourceText, InsertText: string; Position: Integer): string;
function  YearOfDate(xDate: TDateTime): Word;
function  MonthOfDate(xDate: TDateTime): Word;
function  DayOfDate(xDate: TDateTime): Word;
function  AddSlash(Text: string): string;
function  IsKeyDown(VirtualKey: Integer): Boolean;
function  DPH(Cena, Procenta: Currency; Typ: Char): Real;
function  IsInSeason(Year, Month, YearMin, MonthMin, YearMax, MonthMax: Integer): Boolean;
procedure Waiting(Delay: Integer);

implementation

uses  Messages, Graphics, Controls, Forms, Dialogs,
      StdCtrls, Buttons;

constructor TDate.Create;
var
  SystemTime: TSystemTime;
begin
  GetLocalTime(SystemTime);
  FDay:=SystemTime.wDay;
  FMonth:=SystemTime.wMonth;
  FYear:=SystemTime.wYear;
  FLangType:=ltEnglish;
end;

procedure TDate.Assign(Source: TPersistent);
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  if Source is TDate then
    begin
    FDay:=TDate(Source).FDay;
    FMonth:=TDate(Source).FMonth;
    FYear:=TDate(Source).FYear;
    end
  else inherited Assign(Source);
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

procedure TDate.ChangeDate;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TDate.SetDay(Value: Byte);
begin
  if Value=FDay then Exit;
  if Value>DaysInMonth then FDay:=DaysInMonth else if Value<1 then FDay:=1 else FDay:=Value;
  ChangeDate;
end;

procedure TDate.SetLangType(Value: TLangType);
begin
  if Value=FLangType then Exit;
  FLangType:=Value;
end;

procedure TDate.SetMonth(Value: Byte);
begin
  if Value=FMonth then Exit;
  if Value>12 then FMonth:=12 else if Value<1 then FMonth:=1 else FMonth:=Value;
  CheckValidDate;
  ChangeDate;
end;

procedure TDate.SetValue(Value: TDateTime);
begin
  JulianToDate(Trunc(Value-1));
end;

procedure TDate.SetYear(Value: Word);
begin
  if Value=FYear then Exit;
  if Value<1900 then FYear:=1900 else FYear:=Value;
  CheckValidDate;
  ChangeDate;
end;

procedure TDate.CheckValidDate;
begin
  if FYear<1900 then FYear:=1900;
  if FMonth>12 then FMonth:=12 else if FMonth<1 then FMonth:=1;
  if FDay>DaysInMonth then FDay:=DaysInMonth else if FDay<1 then FDay:=1;
end;

function TDate.GetValue: TDateTime;
begin
  Result:=Julian+1;
end;

function TDate.GetLeapYear: Boolean;
begin
  if (FYear mod 4 <> 0) then Result:=False else if (FYear mod 100 <>0) then Result:=True else
    if (FYear mod 400 <> 0) then Result:=False else Result:=True;
end;

procedure TDate.SysDate;
var
  SystemTime: TSystemTime;
begin
  GetLocalTime(SystemTime);
  FDay:=SystemTime.wDay;
  FMonth:=SystemTime.wMonth;
  FYear:=SystemTime.wYear;
end;

procedure TDate.StringToDate(Value: string);
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  if Trim(Copy(Value,7,4))='' then FYear:=1900 else FYear:=StrToInt(Copy(Value,7,4));
  if Trim(Copy(Value,4,2))='' then FMonth:=1 else FMonth:=StrToInt(Copy(Value,4,2));
  if Trim(Copy(Value,1,2))='' then FDay:=1 else FDay:=StrToInt(Copy(Value,1,2));
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

function TDate.DaysInMonth: Byte;
begin
  case FMonth of
    1,3,5,7,8,10,12: Result:=31;
    4,6,9,11: Result:=30;
    2: if LeapYear then Result:=29 else Result:=28;
  else
    Result:=0;
    end;
end;

function TDate.DateToString: string;
begin
  Result:=PadL(Trim(IntToStr(FDay)),2,'0')+DateSeparator+PadL(Trim(IntToStr(FMonth)),2,'0')+DateSeparator+
          PadL(Trim(IntToStr(FYear)),4,'0');
end;

function TDate.DIM(Value: Byte; ValueY: Word): Byte;
begin
  case Value of
    1,3,5,7,8,10,12: Result:=31;
    4,6,9,11: Result:=30;
    2: if LY(ValueY)=366 then Result:=29 else Result:=28;
  else
    Result:=0;
    end;
end;

function TDate.LY(Value: Word): Word;
begin
  if (Value mod 4 <> 0) then
    begin
    Result:=365;
    end
  else
    begin
    if (Value mod 100 <>0) then
      begin
      Result:=366;
      end
    else
      begin
      if (Value mod 400 <> 0) then
        begin
        Result:=365;
        end
      else
        begin
        Result:=366;
        end;
      end;
    end;
end;

function TDate.Julian: Integer;
var i: Integer;
begin
  Result:=0;
  if FYear<1900 then
    begin
    for i:=1899 downto FYear+1 do Dec(Result,LY(i));
    for i:=12 downto FMonth+1 do Dec(Result,DIM(i,FYear));
    Dec(Result,FDay);
    end
  else
    begin
    for i:=1900 to FYear-1 do Inc(Result,LY(i));
    for i:=1 to FMonth-1 do Inc(Result,DIM(i,FYear));
    Inc(Result,FDay);
    end;
end;

function TDate.DayOfWeek: Byte;
begin
  Result:=Julian mod 7;
  Result:=IifInt(Result=0,7,Result);
end;

function TDate.DayOfYear: Word;
var tmpDay, tmpMonth: Integer;
begin
  Result:=Julian;
  tmpDay:=FDay;
  tmpMonth:=FMonth;
  Day:=1;
  Month:=1;
  Result:=Result-Julian+1;
  Day:=tmpDay;
  Month:=tmpMonth;
end;

function TDate.WeekOfYear: Word;
var tmpDay, tmpMonth: Integer;
begin
  Result:=Julian;
  tmpDay:=FDay;
  tmpMonth:=FMonth;
  Day:=1;
  Month:=1;
  Result:=(Result-Julian+1);
  Result:=(Result+(8-DayOfWeek));
  Result:=Result div 7;
  Result:=Result+1;
  Day:=tmpDay;
  Month:=tmpMonth;
end;

function TDate.StrDayOfWeek: string;
begin
  case FLangType of
    ltCzech: Result:=DOWCZ[DayOfWeek];
    else Result:=DOWEN[DayOfWeek];
    end;
end;

function TDate.StrMonth: string;
begin
  case FLangType of
    ltCzech: Result:=MonthCZ[FMonth];
    else Result:=MonthEn[FMonth];
    end;
end;

procedure TDate.Increment;
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  if FDay=DaysInMonth then
    begin
    FDay:=1;
    if FMonth=12 then
      begin
      FMonth:=1;
      Inc(FYear);
      end
    else Inc(FMonth);
    end
  else Inc(FDay);
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

procedure TDate.Decrement;
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  if FDay=1 then
    begin
    FDay:=DaysInMonth;
    if FMonth=1 then
      begin
      FMonth:=12;
      Dec(FYear);
      end
    else Dec(FMonth);
    end
  else Dec(FDay);
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

procedure TDate.IncrementMonth(Count: Integer);
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  Inc(FYear, Count div 12);
  Inc(FMonth, Count mod 12);
  if FMonth>12 then
    begin
    FMonth:=FMonth-12;
    Inc(FYear);
    end;
  SetDay(FDay);
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

procedure TDate.DecrementMonth(Count: Integer);
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  Dec(FYear, Count div 12);
  Dec(FMonth, Count mod 12);
  if FMonth<1 then
    begin
    FMonth:=12-FMonth;
    Dec(FYear);
    end;
  SetDay(FDay);
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

procedure TDate.Add(Count: Integer);
begin
  JulianToDate(Julian+Count);
end;

procedure TDate.Sub(Count: Integer);
begin
  JulianToDate(Julian-Count);
end;

procedure TDate.JulianToDate(JDate: Integer);
var OldYear, OldMonth, OldDay: Integer;
begin
  OldYear:=FYear; OldMonth:=FMonth; OldDay:=Day;
  if JDate<0 then
    begin
    FYear:=1899;
    while JDate<LY(FYear) do
      begin
      JDate:=JDate+LY(FYear);
      Dec(FYear);
      end;
    FMonth:=12;
    while JDate<DIM(FMonth,FYear) do
      begin
      Inc(JDate,DIM(FMonth,FYear));
      Dec(FMonth);
      end;
    FDay:=DIM(FMonth,FYear)-JDate;
    end
  else
    begin
    FYear:=1900;
    while JDate>LY(FYear) do
      begin
      JDate:=JDate-LY(FYear);
      Inc(FYear);
      end;
    FMonth:=1;
    while JDate>DIM(FMonth,FYear) do
      begin
      Dec(JDate,DIM(FMonth,FYear));
      Inc(FMonth);
      end;
    FDay:=JDate;
    end;
  CheckValidDate;
  if (OldYear<>FYear) or (OldMonth<>FMonth) or (OldDay<>FDay) then ChangeDate;
end;

function  TDate.SubDate(XDate: TDate): Integer;
begin
  Result:=Julian-XDate.Julian;
end;

function  TDate.CalendarPoint: TPoint;
var tmpDay: Integer;
begin
  Result.x:=DayOfWeek-1;
  tmpDay:=FDay;
  FDay:=1;
//  Result.y:=(((Julian-FDay+1) mod 7)+FDay-1) div 7;
  Result.y:=((DayOfWeek+tmpDay-2) div 7);
  FDay:=tmpDay;
end;

procedure TDate.SetDateAsCalendarPoint(x, y: Integer);
begin
  FDay:=1;
  JulianToDate(Julian-(DayOfWeek-1)+(y*7)+x);
end;

function  PadC(Text: string; Count: Word; Character: Char): string;
begin
  Result:=Copy(CharCount(Character,Count div 2)+Text+CharCount(Character,(Count div 2)+(Count mod 2)),(Length(CharCount(Character,Count div 2)+Text+CharCount(Character,(Count div 2)+(Count mod 2))) div 2)-(Count div 2),Count);
end;

function  PadL(Text: string; Count: Word; Character: Char): string;
begin
  Result:=Right(CharCount(Character,Count)+Text,Count);
end;

function  PadR(Text: string; Count: Word; Character: Char): string;
begin
  Result:=LeftStr(Text+CharCount(Character,Count),Count);
end;

function  LeftStr(Text: string; Count: Word): string;
begin
  Result:=Copy(Text,1,Count);
end;

function  Right(Text: string; Count: Word): string;
begin
  Result:=Copy(Text,Length(Text)-(Count-1),Count);
end;

function  FindChar(Text: String; Symbol: Char; N: Byte): Integer;
var i, j: Integer;
begin
  J:=0;
  for i:=1 to Length(Text) do
    begin
    if Text[i]=Symbol then j:=j+1;
    if j=N then
      begin
      FindChar:=i;
      Exit;
      end;
    end;
  FindChar:=Length(Text);
end;

function  CountChar(Text: String; Ch: Char): Integer;
var i,j: Integer;
begin
  j:=0;
  for i:=1 to Length(Text) do if Text[i]=Ch then Inc(j);
  CountChar:=j;
end;

function CharCount(Znak: Char; Count: Integer): string;
var i: Integer;
begin
  Result:='';
  for i:=1 to Count do Result:=Result+Znak;
end;

function  MinusStr(Text1, Text2: string): string;
var i: Integer;
begin
  for i:=1 to Length(Text2) do
    if Text1[i]<>Text2[i] then
      begin
      Result:=Copy(Text1,i,Length(Text1)-i+1);
      Exit;
      end;
  Result:=Copy(Text1,Length(Text2)+1,Length(Text1)-Length(Text2)+2);
end;

function IifInt(Podminka: Boolean; Value1, Value2: Integer): Integer;
begin
  if Podminka then IifInt:=Value1 else IifInt:=Value2;
end;

function IifStr(Podminka: Boolean; Value1, Value2: string): string;
begin
  if Podminka then IifStr:=Value1 else IifStr:=Value2;
end;

function CodeString(Text, Code: string): string;
var i, ic, temp: Integer;
    tmpText: string;
begin
  ic:=1;
  tmpText:='';
  Code:=Trim(Code);
  for i:=1 to Length(Text) do
    begin
    temp:=Ord(Text[i])+(Ord(Code[ic])-48);
    tmpText:=tmpText+Chr(IifInt(temp>255,temp-256,temp));
    if ic>=Length(Code) then ic:=1 else Inc(ic);
    end;
  Result:=tmpText;
end;

function DecodeString(Text, Code: string): string;
var i, ic, temp: Integer;
    tmpText: string;
begin
  ic:=1;
  tmpText:='';
  Code:=Trim(Code);
  for i:=1 to Length(Text) do
    begin
    temp:=Ord(Text[i])-(Ord(Code[i])-48);
    tmpText:=tmpText+Chr(IifInt(temp>255,temp-256,temp));
    if ic>=Length(Code) then ic:=1 else Inc(ic);
    end;
  Result:=tmpText;
end;

function  SqrX(Value: Extended; Count: Integer): Extended;
var i: Integer;
begin
  if Count=0 then Result:=1
  else
    begin
    Result:=Value;
    for i:=1 to Count-1 do Result:=Result*Value;
    end;
end;

function  Question(Text: string): Boolean;
var tmpForm: TForm;
    tmpButton1, tmpButton2: TBitBtn;
    tmpLabel: TLabel;
begin
  tmpForm:=TForm.Create(nil);
  tmpForm.Width:=300;
  tmpForm.Height:=100;
  tmpForm.Caption:='';
  tmpForm.BorderStyle:=bsToolWindow;
  tmpForm.Left:=(Screen.Width div 2)-(tmpForm.Width div 2);
  tmpForm.Top:=(Screen.Height div 2)-(tmpForm.Height div 2);
  tmpButton1:=TBitBtn.Create(tmpForm);
  tmpButton1.Parent:=tmpForm;
  tmpButton1.Kind:=bkOK;
  tmpButton1.Caption:='&Ano';
  tmpButton1.Top:=45;
  tmpButton2:=TBitBtn.Create(tmpForm);
  tmpButton2.Parent:=tmpForm;
  tmpButton2.Kind:=bkCancel;
  tmpButton2.Caption:='&Ne';
  tmpButton2.Top:=45;
  tmpLabel:=TLabel.Create(tmpForm);
  tmpLabel.Parent:=tmpForm;
  tmpLabel.Caption:=Text;
  tmpLabel.Left:=20;
  tmpLabel.Top:=10;
  tmpForm.Width:=IifInt((tmpLabel.Width+50)<300,300,tmpLabel.Width+50);
  tmpButton1.Left:=(tmpForm.Width div 2)-90;
  tmpButton2.Left:=(tmpForm.Width div 2)+15;
  Result:=tmpForm.ShowModal=mrOk;
end;

function  StrToInteger(Text: string): Integer;
var i: Integer;
    tmp, tmp1: string;
begin
  tmp:=Trim(Text);
  tmp1:='';
  for i:=1 to Length(tmp) do
    if ((tmp[i]>='0') and (tmp[i]<='9')) or ((tmp[i]='-') and (i=1)) then
      tmp1:=tmp1+tmp[i] else Break;
  if (tmp1='') or (tmp1='-') then Result:=0 else Result:=StrToInt(tmp1);
end;

function  AbsInt(Value: Integer): Integer;
begin
  Result:=Value*IifInt(Value<0,-1,1);
end;

function  InsertStrRight(SourceText, InsertText: string; Position: Integer): string;
begin
  Result:=Copy(SourceText,1,Length(SourceText)-Position)+InsertText+Right(SourceText,Position);
end;

function  InsertStrLeft(SourceText, InsertText: string; Position: Integer): string;
begin
  Result:=Copy(SourceText,1,Position)+InsertText+Right(SourceText,Length(SourceText)-Position);
end;

function  YearOfDate(xDate: TDateTime): Word;
var tmpDay, tmpMonth, tmpYear: Word;
begin
  DecodeDate(xDate,tmpYear,tmpMonth,tmpDay);
  Result:=tmpYear
end;

function  MonthOfDate(xDate: TDateTime): Word;
var tmpDay, tmpMonth, tmpYear: Word;
begin
  DecodeDate(xDate,tmpYear,tmpMonth,tmpDay);
  Result:=tmpMonth
end;

function  DayOfDate(xDate: TDateTime): Word;
var tmpDay, tmpMonth, tmpYear: Word;
begin
  DecodeDate(xDate,tmpYear,tmpMonth,tmpDay);
  Result:=tmpDay
end;

function  AddSlash(Text: string): string;
begin
  if (Length(Text)=0) or (Text[Length(Text)]='\') then Result:=Text else Result:=Text+'\';
end;

function  IsKeyDown(VirtualKey: Integer): Boolean;
begin
  Result:=(GetKeyState(VirtualKey) and 128)=128;
end;

function  DPH(Cena, Procenta: Currency; Typ: Char): Real;
begin
  Result:=0;
  case Typ of
    'D': Result:=Ceil(Round(Cena*(Procenta/(Procenta+100))*100)/10)/10;
    'Z': Result:=Cena-Ceil(Round(Cena*(Procenta/(Procenta+100))*100)/10)/10;
    end;
end;

function  IsInSeason(Year, Month, YearMin, MonthMin, YearMax, MonthMax: Integer): Boolean;
begin
  Result:=False;
  if YearMin=0 then
    begin
    if YearMax=0 then
      begin
      if Year<=YearMax then Result:=True;
      end
    else
      begin
      if (Year<=YearMax) and (Month<=MonthMax) then Result:=True;
      end;
    end
  else
    begin
    if YearMax=0 then
      begin
      if MonthMax=0 then
        begin
        if Year>=YearMin then Result:=True;
        end
      else
        begin
        if (Year>=YearMin) and (Month>=MonthMin) then Result:=True;
        end;
      end
    else
      begin
      if MonthMax=0 then
        begin
        if MonthMin=0 then
          begin
          if (Year>=YearMin) and (Year<=YearMax) then Result:=True;
          end
        else
          begin
          if (Year>=YearMin) and (Year<=YearMax) and (Month>=MonthMin) then Result:=True;
          end;
        end
      else
        begin
        if MonthMin=0 then
          begin
          if (Year>=YearMin) and (Year<=YearMax) and (Month<=MonthMax) then Result:=True;
          end
        else
          begin
          if (Year>=YearMin) and (Year<=YearMax) and (Month>=MonthMin) and (Month<=MonthMax) then Result:=True;
          end;
        end;
      end;
    end;
end;

procedure Waiting(Delay: Integer);
var tmpCas: Cardinal;
begin
  tmpCas:=GetCurrentTime;
  while tmpCas+Delay>GetCurrentTime do;
end;

end.

